@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/choice.c,v 4.44 2002/01/05 02:47:59 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/choice.c,v 1.12 2009/08/04 04:39:24 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_CHOICE_C_
@@ -147,6 +147,8 @@
 int	curChoice=NOTHING;
 int	cycleThroughChoice=FALSE;
 
+int	setCurChoiceDontUpdateUI=FALSE;
+
 static int	curRaisedChoice=INVALID;
 static int	curRaisedMode=INVALID;
 
@@ -482,6 +484,11 @@ void ShowCurFont()
    recs[0].height = choiceWindowH;
    XSetClipRectangles(mainDisplay, choiceGC, 0, 0, recs, 1, YXBanded);
 
+   /*
+    * The following code was added so that when you move the text cursor
+    *         from a double-byte font to a single-byte font, we can close
+    *         the double-byte input method.
+    */
    if (gnInputMethod != TGIM_NONE) {
       if (snPrevShowDoubleByteFont != INVALID &&
             snPrevShowDoubleByteFont && !canvasFontDoubleByte &&
@@ -961,6 +968,7 @@ void SetCurChoice(NewChoice)
    int NewChoice;
 {
    struct ObjRec *selected_single_text_obj=NULL;
+   int old_choice=curChoice, sel_top_obj=FALSE;
 
    if (curChoice == NewChoice) return;
    if (gstWBInfo.do_whiteboard && (serializingFile || deserializingFile)) {
@@ -986,6 +994,8 @@ void SetCurChoice(NewChoice)
             }
             HighLightReverse();
             RemoveAllSel();
+         } else {
+            HighLightReverse();
          }
       }
       break;
@@ -993,6 +1003,7 @@ void SetCurChoice(NewChoice)
       CreateTextObj(TRUE, TRUE);
       if (NeedSelectTopObject(NewChoice, FALSE) && textDrawn) {
          HighLightJustDrawnText();
+         sel_top_obj = TRUE;
       }
       textDrawn = FALSE;
       textCursorShown = FALSE;
@@ -1000,6 +1011,7 @@ void SetCurChoice(NewChoice)
    case DRAWBOX:
       if (NeedSelectTopObject(NewChoice, FALSE) && boxDrawn) {
          SelectTopObj();
+         sel_top_obj = TRUE;
       }
       boxDrawn = FALSE;
       break;
@@ -1008,18 +1020,21 @@ void SetCurChoice(NewChoice)
    case DRAWEDGECIRCLE:
       if (NeedSelectTopObject(NewChoice, FALSE) && ovalDrawn) {
          SelectTopObj();
+         sel_top_obj = TRUE;
       }
       ovalDrawn = FALSE;
       break;
    case DRAWPOLY:
       if (NeedSelectTopObject(NewChoice, TRUE) && polyDrawn) {
          SelectTopObj();
+         sel_top_obj = TRUE;
       }
       polyDrawn = FALSE;
       break;
    case DRAWPOLYGON:
       if (NeedSelectTopObject(NewChoice, TRUE) && polygonDrawn) {
          SelectTopObj();
+         sel_top_obj = TRUE;
       }
       polygonDrawn = FALSE;
       break;
@@ -1027,26 +1042,32 @@ void SetCurChoice(NewChoice)
    case DRAWEDGEARC:
       if (NeedSelectTopObject(NewChoice, FALSE) && arcDrawn) {
          SelectTopObj();
+         sel_top_obj = TRUE;
       }
       arcDrawn = FALSE;
       break;
    case DRAWRCBOX:
       if (NeedSelectTopObject(NewChoice, FALSE) && rcBoxDrawn) {
          SelectTopObj();
+         sel_top_obj = TRUE;
       }
       rcBoxDrawn = FALSE;
       break;
    case FREEHAND:
       if (NeedSelectTopObject(NewChoice, TRUE) && polyDrawn) {
          SelectTopObj();
+         sel_top_obj = TRUE;
       }
       polyDrawn = FALSE;
       break;
    case VERTEXMODE:
-      if (NewChoice == NOTHING || NewChoice == ROTATEMODE) {
+      if (NewChoice == NOTHING) {
          HighLightReverse();
          JustRemoveAllVSel();
          HighLightForward();
+      } else if (NewChoice == ROTATEMODE) {
+         HighLightReverse();
+         JustRemoveAllVSel();
       } else {
          HighLightReverse();
          RemoveAllSel();
@@ -1057,6 +1078,7 @@ void SetCurChoice(NewChoice)
          if (NewChoice == VERTEXMODE) {
             UnSelNonVertexObjs(TRUE, TRUE); /* with highlight */
             UpdSelBBox();
+            HighLightReverse();
          } else if (NewChoice != NOTHING) {
             if (NewChoice == DRAWTEXT && topSel != NULL && topSel == botSel &&
                   topSel->obj->type == OBJ_TEXT) {
@@ -1064,6 +1086,8 @@ void SetCurChoice(NewChoice)
             }
             HighLightReverse();
             RemoveAllSel();
+         } else {
+            HighLightReverse();
          }
       }
       break;
@@ -1093,17 +1117,39 @@ void SetCurChoice(NewChoice)
          }
       }
       break;
-   case NOTHING: break;
-   case VERTEXMODE: break;
-   case ROTATEMODE: break;
+   case NOTHING:
+      if (old_choice != VERTEXMODE && !sel_top_obj) {
+         HighLightForward();
+      }
+      break;
+   case VERTEXMODE:
+      if (old_choice == ROTATEMODE) {
+         HighLightForward();
+      }
+      break;
+   case ROTATEMODE:
+      if (sel_top_obj) {
+         curChoice = old_choice;
+         HighLightReverse();
+         curChoice = ROTATEMODE;
+         HighLightForward();
+      } else if (old_choice == NOTHING) {
+         HighLightForward();
+      } else if (old_choice == VERTEXMODE) {
+         HighLightForward();
+      }
+      break;
+   }
+   if (!setCurChoiceDontUpdateUI) {
+      ShowCursor();
+
+      RedrawModeWindow();
+      UpdatePinnedMenu(MENU_MODE);
+
+      ShowCurChoiceMouseStatus(curChoice, 0, FALSE);
    }
-   ShowCursor();
    textCursorShown = FALSE;
 
-   RedrawModeWindow();
-   UpdatePinnedMenu(MENU_MODE);
-
-   ShowCurChoiceMouseStatus(curChoice, 0, FALSE);
    if (inHyperSpace && curChoice != NOTHING) {
       ToggleHyperSpace(FALSE);
    } else if (!inHyperSpace && selected_single_text_obj != NULL) {
@@ -1790,6 +1836,9 @@ int ChoiceEventHandler(input)
                case LT_INTSPLINE:
                   Msg(TgLoadString(STID_LINE_TYPE_IS_INTSPLINE));
                   break;
+               case LT_STRUCT_SPLINE:
+                  Msg(TgLoadString(STID_LINE_TYPE_IS_STRUCT_SPLINE));
+                  break;
                }
             }
             QuickSetMouseStatus(lineTypeMouseStatus, 0);
@@ -2046,21 +2095,37 @@ int ModeEventHandler(input)
       if (mode >=0 && mode < MAXCHOICES) {
          TgMenuItemInfo *menu_iteminfos=modeMenuInfo.items;
 
-         /* the raw status_str is English */
-         SetStringStatus(_(menu_iteminfos[mode].status_str));
+         if (mode == ROTATEMODE) {
+            QuickSetMouseStatus(autoRotatePivotMouseStatus, 0);
+         } else {
+            /* the raw status_str is English */
+            SetStringStatus(_(menu_iteminfos[mode].status_str));
+         }
       } else {
          SetStringStatus("");
       }
    } else if (input->type == ButtonPress) {
+      XButtonEvent *button_ev=(&(input->xbutton));
+
       mode = (int)((input->xbutton.y-(windowPadding>>1)) /
             (choiceImageH+(windowPadding<<1)));
       if (mode >=0 && mode < MAXCHOICES) {
-         TgMenuItemInfo *menu_iteminfos=modeMenuInfo.items;
+         if (button_ev->button == Button2 && mode == ROTATEMODE) {
+            int x=0, y=0, root_x=0, root_y=0;
+            Window root_win=None, child_win=None;
+            unsigned int status=0;
 
-         SetCurChoice(mode);
-         /* the raw status_str is English */
-         SetStringStatus(_(menu_iteminfos[mode].status_str));
-      } else if (input->xbutton.button == Button2 && mode >= MAXCHOICES) {
+            XQueryPointer(mainDisplay, rootWindow, &root_win, &child_win,
+                  &root_x, &root_y, &x, &y, &status);
+            AutoRotatePivotMenu(x, y, FALSE);
+         } else {
+            TgMenuItemInfo *menu_iteminfos=modeMenuInfo.items;
+
+            SetCurChoice(mode);
+            /* the raw status_str is English */
+            SetStringStatus(_(menu_iteminfos[mode].status_str));
+         }
+      } else if (button_ev->button == Button2 && mode >= MAXCHOICES) {
          return MainMenu();
       }
    }
