@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/const.h,v 4.43 2002/01/05 02:47:59 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/const.h,v 1.20 2009/07/28 14:33:55 william Exp $
  */
 
 #ifndef _TGIF_CONST_H_
@@ -172,12 +172,17 @@
 
 #define MARK(W,GC,X,Y) XFillRectangle(mainDisplay,W,GC, \
       (X)-handleSize,(Y)-handleSize,(handleSize<<1)+1,(handleSize<<1)+1)
+#define MARKHR(W,GC,X,Y) XDrawRectangle(mainDisplay,W,GC, \
+      (X)-handleSize,(Y)-handleSize,(handleSize<<1),(handleSize<<1))
 #define MARKV(W,GC,X,Y) XDrawLine(mainDisplay,W,GC, \
       (X)-(handleSize+1),(Y),(X)+(handleSize+1),(Y)); \
       XDrawLine(mainDisplay,W,GC,(X),(Y)-(handleSize+1),(X),(Y)+(handleSize+1))
 #define MARKO(W,GC,X,Y) XFillArc(mainDisplay,W,GC, \
       (X)-handleSize,(Y)-handleSize,(handleSize<<1)+1,(handleSize<<1)+1, \
       0,(360<<6))
+#define MARKHO(W,GC,X,Y) XDrawArc(mainDisplay,W,GC, \
+      (X)-handleSize,(Y)-handleSize,(handleSize<<1),(handleSize<<1), \
+      0,(360<<6))
 #define PtInMark(PtX,PtY,MarkX,MarkY) ((PtX) >= (MarkX)-handleSize && \
       (PtY) >= (MarkY)-handleSize && (PtX) <= (MarkX)+handleSize && \
       (PtY) <= (MarkY)+handleSize)
@@ -187,6 +192,7 @@
 #define GETFLT(category,val,name) ScanValue("%f", &(val), name, category)
 #define GETDBL(category,val,name) ScanValue("%lg", &(val), name, category)
 #define GETSTR(category,val,name) ScanValue("%s", (val), name, category)
+#define GETDYN(category,val,name) ScanDynStrValue(&(val), name, category)
 
 #define GetPinObj(ObjPtr) ((ObjPtr)->detail.r->pin_connected ? \
       (ObjPtr)->detail.r->last : (ObjPtr)->detail.r->first)
@@ -272,8 +278,9 @@
 #define LT_STRAIGHT 0
 #define LT_SPLINE 1
 #define LT_INTSPLINE 2
+#define LT_STRUCT_SPLINE 3
 
-#define MAXLINETYPES 3
+#define MAXLINETYPES 4
 
 #define LS_PLAIN 0
 #define LS_RIGHT 1
@@ -451,9 +458,11 @@
 #define TIFFEPSI_FILE 9
 #define PNG_FILE 10
 #define JPEG_FILE 11
-#define NETLIST_FILE 12
+#define PPM_FILE 12
+#define NETLIST_FILE 13
+#define SVG_FILE 14
 
-#define MAXDEFWHERETOPRINT 13
+#define MAXDEFWHERETOPRINT 15
 
 /* measurement */
 
@@ -690,19 +699,32 @@
 #define XBM_EPS 1
 #define XBM_EPSI 2
 
+/* xpm real_type */
+
+#define XPM_XPM  0
+#define XPM_JPEG 1
+#define PPM_TRUE 2
+#define XPM_EPS  3 /* not implemented */
+#define XPM_EPSI 4 /* not implemented */
+
+#define PPM_DATA_RAW      0 /* not used */
+#define PPM_JPEG_COMPRESS 1
+#define PPM_DATA_DEFLATED 2
+
 /* cmd stuff */
 
-#define CMD_COMPOSITE	 0
-#define CMD_NEW		 1
-#define CMD_DELETE	 2
-#define CMD_MOVE	 3
-#define CMD_STRETCH	 4
-#define CMD_ONE_TO_MANY	 5
-#define CMD_MANY_TO_ONE	 6
-#define CMD_REPLACE	 7
-#define CMD_GOTO_PAGE	 8
-#define CMD_WB_CLEARALL	 9
-#define CMD_CHAT_A_LINE	10
+#define CMD_COMPOSITE     0
+#define CMD_NEW           1
+#define CMD_DELETE        2
+#define CMD_MOVE          3
+#define CMD_STRETCH       4
+#define CMD_ONE_TO_MANY   5
+#define CMD_MANY_TO_ONE   6
+#define CMD_REPLACE       7
+#define CMD_GOTO_PAGE     8
+#define CMD_WB_CLEARALL   9
+#define CMD_CHAT_A_LINE  10
+#define CMD_WB_SLIDESHOW 11
 
 /* rcb radius stuff */
 
@@ -713,7 +735,8 @@
 
 #define ITEM_DSPED 10
 #define ITEM_LEN 30
-#define ROW_HEIGHT ((msgFontPtr==NULL)?(defaultFontHeight+1):(msgFontHeight+1))
+#define ROW_HEIGHT \
+  ((msgFontSet==NULL&&msgFontPtr==NULL)?(defaultFontHeight+1):(msgFontHeight+1))
 #define BUTTON_OK 0
 #define BUTTON_SETDIR 1
 #define BUTTON_CANCEL 2
@@ -730,6 +753,7 @@
 /* copypaste stuff */
 
 #define TGIF_HEADER 0x80
+#define TGIF_PROTOCOL_ATOM "_TGIF_PROTOCOL_ATOM"
 
 /* interrupt stuff */
 
@@ -826,6 +850,10 @@
 #define TGIM_CHINPUT    1
 #define TGIM_KINPUT     2
 #define TGIM_XIM        3
+#define TGIM_TGTWB5     4
+
+#define TGIM_DBIM       0x1
+#define TGIM_SBIM       0x2
 
 /* postscript stuff */
 
@@ -957,4 +985,16 @@
 #define PROP_MASK_UNDERLINE_ON	0x00020000
 #define PROP_MASK_WIDTH_INDEX	0x00100000 /* GetProperty() only */
 
+/* mark stuff */
+
+#define FORWARD 0
+#define REVERSE 1
+
+/* auto rotate pivot stuff */
+
+#define AUTO_ROTATE_PIVOT 0
+#define USER_ROTATE_PIVOT 1
+
+#define MAX_ROTATE_PIVOT 2
+
 #endif /*_TGIF_CONST_H_*/
