@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/cursor.c,v 4.10 2001/08/06 21:39:00 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/cursor.c,v 1.6 2009/01/09 22:25:29 william Exp $
  */
 
 #define _INCLUDE_FROM_CURSOR_C_
@@ -25,6 +25,7 @@
 #include "choice.e"
 #include "color.e"
 #include "cursor.e"
+#include "file.e"
 #include "setup.e"
 #include "strtbl.e"
 #include "xbitmap.e"
@@ -252,9 +253,11 @@ void SetNullCursor(window)
 void SetWatchCursor(window)
    Window window;
 {
-   XDefineCursor(mainDisplay, window, watchCursor);
-   if (window == mainWindow) watchCursorOnMainWindow = TRUE;
-   XSync(mainDisplay, False);
+   if (!PRTGIF || cmdLineOpenDisplay) {
+      XDefineCursor(mainDisplay, window, watchCursor);
+      if (window == mainWindow) watchCursorOnMainWindow = TRUE;
+      XSync(mainDisplay, False);
+   }
 }
 
 void SetDrawCursor(window)
@@ -332,9 +335,11 @@ void SetHandCursor(window)
 void SetDefaultCursor(window)
    Window window;
 {
-   XDefineCursor(mainDisplay, window, defaultCursor);
-   if (window == mainWindow && watchCursorOnMainWindow) {
-      watchCursorOnMainWindow = FALSE;
+   if (!PRTGIF || cmdLineOpenDisplay) {
+      XDefineCursor(mainDisplay, window, defaultCursor);
+      if (window == mainWindow && watchCursorOnMainWindow) {
+         watchCursorOnMainWindow = FALSE;
+      }
    }
 }
 
