@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/grid.c,v 4.45 2002/03/21 07:44:50 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/grid.c,v 1.7 2009/06/05 21:52:40 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_GRID_C_
@@ -49,6 +49,7 @@
 #include "strtbl.e"
 #include "text.e"
 #include "util.e"
+#include "wb.e"
 #include "xprtfltr.e"
 
 #define ENGLISH_GRID_STEP 8
@@ -547,9 +548,14 @@ void WhereToPrintMsg()
          break;
       case PNG_FILE: Msg(TgLoadString(STID_WILL_EXPORT_PNG_FILE)); break;
       case JPEG_FILE: Msg(TgLoadString(STID_WILL_EXPORT_JPEG_FILE)); break;
+      case PPM_FILE:
+         Msg(TgLoadString(colorDump ? STID_WILL_EXPORT_PPM_FILE :
+               STID_WILL_EXPORT_PBM_FILE));
+         break;
       case NETLIST_FILE:
          Msg(TgLoadString(STID_WILL_EXPORT_NETLIST_FILE));
          break;
+      case SVG_FILE: Msg(TgLoadString(STID_WILL_EXPORT_SVG_FILE)); break;
       }
    }
 }
@@ -1369,6 +1375,13 @@ int RefreshShowHideMenu(menu)
    /* ShowMode */
    ok &= TgSetMenuItemCheckById(menu, CMDID_TOGGLESHOWMODE, !noModeWindow);
 
+   if (gstWBInfo.do_whiteboard) {
+      /* ShowChat */
+      ok &= TgEnableMenuItemById(menu, CMDID_TOGGLESHOWCHAT, TRUE);
+      ok &= TgSetMenuItemCheckById(menu, CMDID_TOGGLESHOWCHAT, !noChatWindow);
+   } else {
+      ok &= TgEnableMenuItemById(menu, CMDID_TOGGLESHOWCHAT, FALSE);
+   }
    return ok;
 }
 
