@@ -5,7 +5,7 @@
  *
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -19,7 +19,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/http.c,v 4.21 2001/08/06 21:39:04 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/http.c,v 1.10 2009/09/28 22:39:48 william Exp $
  */
 
 #define _INCLUDE_FROM_HTTP_C_
@@ -132,11 +132,13 @@ static int gnDecode[] = {
    41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, -1, -1, -1, -1, -1
 };
 
-static
-void DoBase64Encode(buf, buf_len, return_buf)
-   char *buf, *return_buf;
+void DoBase64Encode(buf_in, buf_len, return_buf_in)
+   char *buf_in, *return_buf_in;
    int buf_len;
 {
+   unsigned char *buf=(unsigned char *)buf_in;
+   unsigned char *return_buf=(unsigned char *)return_buf_in;
+
    while (buf_len > 0) {
       switch (buf_len) {
       case 1:
@@ -186,6 +188,84 @@ char *Base64Encode(buf)
    return return_buf;
 }
 
+static
+int Convert(buf4, count, buf3)
+   int *buf4, count;
+   char *buf3;
+{
+   switch (count) {
+   case 2:
+      buf3[0] = ((buf4[0]<<2) & 0xfc) | ((buf4[1]>>4) & 0x03);
+      buf3[1] = '\0';
+      return 1;
+   case 3:
+      buf3[0] = ((buf4[0]<<2) & 0xfc) | ((buf4[1]>>4) & 0x03);
+      buf3[1] = ((buf4[1]<<4) & 0xf0) | ((buf4[2]>>2) & 0x0f);
+      buf3[2] = '\0';
+      return 2;
+   case 4:
+      buf3[0] = (char)(((buf4[0]<<2) & 0xfc) | ((buf4[1]>>4) & 0x03));
+      buf3[1] = (char)(((buf4[1]<<4) & 0xf0) | ((buf4[2]>>2) & 0x0f));
+      buf3[2] = (char)(((buf4[2]<<6) & 0xc0) | (buf4[3] & 0x3f));
+      buf3[3] = '\0';
+      return 3;
+   default:
+#ifdef _TGIF_DBG /* debug, do not translate */
+      Error("Convert()", "Bad count passed to Convert().");
+#endif /* _TGIF_DBG */
+      break;
+   }
+   return 0;
+}
+
+int DoBase64Decode(buf, buf_sz, return_buf)
+   char *buf, *return_buf;
+   int buf_sz;
+{
+   int i=0, j=0, bytes_decoded=0, intbuf[5];
+
+   intbuf[4] = (-1);
+   for (i=0, j=0; i < buf_sz; i++) {
+      intbuf[j++] = gnDecode[(int)buf[i]];
+      if (j == 4) {
+         char out_buf[4];
+         int k=0, bytes=0;
+
+         if (intbuf[3] == (-1)) {
+            if (intbuf[2] == (-1)) {
+               bytes = Convert(intbuf, 2, out_buf);
+            } else {
+               bytes = Convert(intbuf, 3, out_buf);
+            }
+         } else {
+            bytes = Convert(intbuf, 4, out_buf);
+         }
+         for (k=0; k < bytes; k++) {
+            *return_buf++ = (char)(out_buf[k] & 0x0ff);
+         }
+         j = 0;
+         bytes_decoded += bytes;
+      }
+   }
+   *return_buf = '\0';
+   return bytes_decoded;
+}
+
+char *Base64Decode(buf)
+   char *buf;
+{
+   int len=strlen(buf);
+   char *return_buf=(char*)malloc(len*sizeof(char));
+
+   if (return_buf == NULL) {
+      FailAllocMessage();
+      return NULL;
+   }
+   (void)DoBase64Decode(buf, len, return_buf);
+
+   return return_buf;
+}
+
 static
 void BubbleAuthInfoToTop(pai)
    struct AuthInfo *pai;
@@ -370,8 +450,6 @@ void CleanUpHttp()
 
    UtilFree(gpszFakedReferer);
    gpszFakedReferer = NULL;
-
-   if (gnDecode) { }
 }
 
 void InitHttp()
@@ -746,6 +824,17 @@ int HttpDoWrite(n_socket, psz_path, psz_host, us_port)
    sprintf(msg, TgLoadCachedString(CSTID_SENDING_REQUESTS_DOTS), "HTTP");
    ShowRemoteStatus(msg);
 
+   if (debugHttp == 999 || ((debugHttp % 100) == 99 && cmdLineDumpURL &&
+         cmdLineDumpURLWithHeader)) {
+      /* debug, do not translate */
+      fprintf(stderr, "************************\n");
+      fprintf(stderr, "* Begin Request Header *\n");
+      fprintf(stderr, "************************\n");
+      (void)fwrite(buf, sizeof(char), strlen(buf), stderr);
+      fprintf(stderr, "************************\n");
+      fprintf(stderr, "*  End Request Header  *\n");
+      fprintf(stderr, "************************\n");
+   }
    status = TcpDoWrite(n_socket, buf, (int)strlen(buf));
    free(buf);
 
@@ -756,6 +845,7 @@ int HttpDoWrite(n_socket, psz_path, psz_host, us_port)
    return status;
 }
 
+#ifdef NOT_DEFINED
 #ifdef _TGIF_DBG /* debug, do not translate */
 static char SZ_DEF_GIF_NAME[]="/tmp/htclient.gif";
 
@@ -766,7 +856,7 @@ void DebugHttpDumpResponse(buf)
    char *c_ptr=strchr(buf, '\n'), *line_ptr=buf, *content_type=NULL;
    int content_length=(-1), gif87a=FALSE;
    int len1=strlen(SZ_CONTENT_TYPE), len2=strlen(SZ_CONTENT_LENGTH);
-   FILE *fp=stdout;
+   FILE *fp=stderr;
 
    while (c_ptr != NULL) {
       char *prev_ptr=c_ptr;
@@ -840,6 +930,7 @@ void DebugHttpDumpResponse(buf)
    }
 }
 #endif /* _TGIF_DBG */
+#endif /* NOT_DEFINED */
 
 static
 int ParseBadFormat(buf)
@@ -1010,7 +1101,7 @@ char *HttpExtractText(buf, pn_buf_sz, pn_html, ppsz_content_type)
    char *c_ptr=strchr(buf, '\n'), *line_ptr=buf, *content_type=NULL;
    int content_length=(-1), text_type=FALSE, first_line=TRUE;
    int len1=strlen(SZ_CONTENT_TYPE), len2=strlen(SZ_CONTENT_LENGTH);
-   FILE *fp=stdout;
+   FILE *fp=stderr;
 
    CleanUpHttpHeaderInfo();
 
@@ -1028,7 +1119,8 @@ char *HttpExtractText(buf, pn_buf_sz, pn_html, ppsz_content_type)
          *c_ptr = '\0';
       }
       if (debugHttp > 0) {
-         if (debugHttp == 99 && cmdLineDumpURL && !cmdLineDumpURLWithHeader) {
+         if ((debugHttp % 100) == 99 && cmdLineDumpURL &&
+               !cmdLineDumpURLWithHeader) {
          } else if (!deserializingFile) {
             fprintf(fp, "%s\n", line_ptr);
          }
@@ -1199,10 +1291,15 @@ int ReadFromSocket(n_socket)
       status = select(nfds, &fdset, NULL, NULL, &timeout);
 #endif /* __hpux */
       if (status < 0) {
-         sprintf(gszMsgBox, TgLoadString(STID_FUNC_SELECT_SYS_CALL_FAILED),
-               "ReadFromSocket()"),
-         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-         break;
+         if (errno == EINTR) {
+            /* interrupted by a system call, not a problem */
+            CheckInterrupt(TRUE);
+         } else {
+            sprintf(gszMsgBox, TgLoadString(STID_FUNC_SELECT_SYS_CALL_FAILED),
+                  "ReadFromSocket()"),
+            MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+            break;
+         }
       } else if (status == 0) {
          /* spin the interrupt thing */
          CheckInterrupt(TRUE);
@@ -1221,7 +1318,7 @@ int ReadFromSocket(n_socket)
          return TRUE;
       } else {
          sprintf(gszMsgBox, TgLoadString(STID_FUNC_INVALID_RC_FOR_SELECT),
-               "WaitForEvent()", status);
+               "ReadFromSocket()", status);
          MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
          break;
       }
@@ -1343,12 +1440,12 @@ int HttpDoRead(n_socket, ppsz_buf, pn_buf_sz)
       *ppsz_buf = buf;
       if (pn_buf_sz != NULL) *pn_buf_sz = len;
       sprintf(msg, TgLoadCachedString(CSTID_RESPONSES_RECEIVED), "HTTP");
-      if (debugHttp == 99) {
+      if ((debugHttp % 100) == 99) {
          /* debug, do not translate */
          if (!cmdLineDumpURL) {
-            fprintf(stdout, "\n==========>>>\n");
-            fwrite(buf, sizeof(char), len, stdout);
-            fprintf(stdout, "\n<<<==========\n");
+            fprintf(stderr, "\n==========>>>\n");
+            (void)fwrite(buf, sizeof(char), len, stderr);
+            fprintf(stderr, "\n<<<==========\n");
          }
       }
    } else {
