@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/mainloop.c,v 4.96 2002/03/08 00:28:35 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/mainloop.c,v 1.34 2009/10/02 20:14:03 william Exp $
  */
 
 #define _INCLUDE_FROM_MAINLOOP_C_
@@ -30,7 +30,9 @@
 #endif /* USE_XT_INITIALIZE */
 
 #include "animate.e"
+#include "attr.e"
 #include "auxtext.e"
+#include "chat.e"
 #include "choice.e"
 #include "cmd.e"
 #include "color.e"
@@ -78,6 +80,8 @@
 #include "strtbl.e"
 #include "tangram2.e"
 #include "text.e"
+#include "tgcwheel.e"
+#include "tidget.e"
 #include "util.e"
 #include "version.e"
 #include "wb.e"
@@ -85,11 +89,9 @@
 #include "xpixmap.e"
 #include "xprtfltr.e"
 
-#ifndef USE_XAPPLRESDIR
 #ifdef USE_XT_INITIALIZE
 Widget		toplevel=(Widget)0;
 #endif /* USE_XT_INITIALIZE */
-#endif /* ~USE_XAPPLRESDIR */
 
 int	x11R6OrAbove=FALSE;
 int	origArgC=0;
@@ -112,6 +114,10 @@ char	*displayName=NULL;
 int	initializingMain=FALSE;
 int	cmdLineUseXlib=FALSE;
 int	cmdLineVersion=FALSE;
+int	cmdLineNoModeWindow=FALSE;
+int	cmdLineNoChatWindow=FALSE;
+
+KeyOrBtnEvInfo	lastKeyOrBtnEvInfo;
 
 int			numExtraWins=0;
 struct WinInfoRec	*extraWinInfo=NULL;
@@ -120,9 +126,6 @@ PageInfo	mergingPageInfo;
 
 static int	maxExtraWins=0;
 static int	quitDraw=TRUE;
-#ifdef USE_XAPPLRESDIR
-static char	*xEnvStr=NULL;
-#endif /* USE_XAPPLRESDIR */
 
 static int	useXlib=FALSE;
 
@@ -184,16 +187,25 @@ void CleanUpExtraWinInfo()
    extraWinInfo = NULL;
 }
 
+static
+void InitLastKeyOrBtnEvInfo()
+{
+   memset(&lastKeyOrBtnEvInfo, 0, sizeof(KeyOrBtnEvInfo));
+}
+
 void CleanUpTalkToSelfFiledes()
 {
-   if (talkToSelfFiledes[0] != (-1)) close(talkToSelfFiledes[0]);
-   if (talkToSelfFiledes[1] != (-1)) close(talkToSelfFiledes[1]);
-
+   if (talkToSelfFiledesInitialized) {
+       if (talkToSelfFiledes[0] != (-1)) close(talkToSelfFiledes[0]);
+       if (talkToSelfFiledes[1] != (-1)) close(talkToSelfFiledes[1]);
+   }
+   talkToSelfFiledesInitialized = FALSE;
    talkToSelfFiledes[0] = talkToSelfFiledes[1] = (-1);
 }
 
 int InitTalkToSelfFiledes()
 {
+   talkToSelfFiledesInitialized = FALSE;
    talkToSelfFiledes[0] = talkToSelfFiledes[1] = (-1);
 
    if (pipe(talkToSelfFiledes) != 0) {
@@ -201,6 +213,8 @@ int InitTalkToSelfFiledes()
       fprintf(stderr, "%s\n", gszMsgBox);
       return FALSE;
    }
+   talkToSelfFiledesInitialized = TRUE;
+
    return TRUE;
 }
 
@@ -226,7 +240,7 @@ void ExecWithFile(CmdName, FileName)
    /* do not translate -- program constants */
    sprintf(s, "xterm -bd red -e %s %s", CmdName, FileName);
    strcat(s, " &");
-   system(s);
+   (void)system(s);
 #else /* ~_BACKGROUND_DONT_FORK */
    int pid;
 
@@ -234,7 +248,7 @@ void ExecWithFile(CmdName, FileName)
    sprintf(s, "xterm -bd red -e %s %s", CmdName, FileName);
    pid = fork();
    if (pid == 0) {
-      system(s);
+      (void)system(s);
       exit(0);
    }
 #endif /* _BACKGROUND_DONT_FORK */
@@ -304,14 +318,14 @@ void CleanUpMergeArgs()
 void CleanUp()
 {
    tgCleanUpProfile();
+   CleanUpChat();
    CleanUpWhiteBoard();
 
    TieLooseEnds();
 
-#ifdef _TANGRAM_II
-   CleanUpTangram2();
-#endif /* _TANGRAM_II */
-
+   if (cmdLineTgrm2) {
+      CleanUpTangram2();
+   }
    CleanUpMiniLines();
    CleanUpCutPaste();
    CleanUpProperties();
@@ -360,7 +374,10 @@ void CleanUp()
    CleanUpRecentFiles();
    CleanUpMeasureTooltip();
    CleanUpSpecial();
+   CleanUpAttr();
    CleanUpTalkToSelfFiledes();
+   CleanUpColorWheel();
+   CleanUpTidget();
 
    if (iconWindowCreated) {
       XDestroyWindow(mainDisplay, iconBaseWindow);
@@ -383,8 +400,9 @@ void ExposeEventHandler(input, recursive)
    XEvent *input;
    int recursive;
 {
-   register int i;
+   int i=0;
    XEvent ev;
+   TidgetInfo *handling_pti=NULL;
 
    if (input->xany.window == choiceWindow) {
       while (XCheckWindowEvent(mainDisplay, choiceWindow, ExposureMask, &ev)) ;
@@ -428,6 +446,8 @@ void ExposeEventHandler(input, recursive)
    } else if (input->xany.window == statusWindow) {
       while (XCheckWindowEvent(mainDisplay, statusWindow, ExposureMask, &ev)) ;
       RedrawStatusWindow();
+   } else if (IsChatWindowEvent(input, &handling_pti)) {
+      ChatEventHandler(input, handling_pti);
    } else if (input->xany.window == modeWindow) {
       while (XCheckWindowEvent(mainDisplay, modeWindow, ExposureMask, &ev)) ;
       RedrawModeWindow();
@@ -459,6 +479,8 @@ void ExposeEventHandler(input, recursive)
             }
          }
       }
+   } else if (TidgetManagerHandleEvent(input)) {
+      /* input has been handled by the TidgetManager */
    } else {
       for (i = 0; i < numExtraWins; i++) {
          if (input->xany.window == extraWinInfo[i].window &&
@@ -483,7 +505,7 @@ int GzipFile(fname, gzipped_fname)
 {
    int rc=TRUE;
    FILE *pfp=NULL, *out_fp=NULL;
-   char *compress_cmd=gunzipCmd, *cmd=NULL;
+   char *compress_cmd=NULL, *cmd=NULL;
 
    if ((out_fp=fopen(gzipped_fname, "w")) == NULL) {
       return FailToOpenMessage(gzipped_fname, "w", NULL);
@@ -499,6 +521,12 @@ int GzipFile(fname, gzipped_fname)
    } else {
       sprintf(cmd, compress_cmd, fname);
    }
+   if (!FindProgramInPath(cmd, NULL, FALSE)) {
+      fclose(out_fp);
+      free(cmd);
+      RestoreStatusStrings();
+      return FALSE;
+   }
    sprintf(gszMsgBox, TgLoadCachedString(CSTID_EXECUTING_GIVEN_PROGRAM),
          gzipCmd);
    SetStringStatus(gszMsgBox);
@@ -509,8 +537,12 @@ int GzipFile(fname, gzipped_fname)
       fclose(out_fp);
       rc = FALSE;
    } else {
-      int bytes_read=0;
+      int bytes_read=0, watch_cursor=watchCursorOnMainWindow;
 
+      if (!watch_cursor) {
+         SetWatchCursor(drawWindow);
+         SetWatchCursor(mainWindow);
+      }
       writeFileFailed = FALSE;
       while ((bytes_read=fread(gszMsgBox, sizeof(char), sizeof(gszMsgBox),
             pfp)) > 0) {
@@ -520,6 +552,10 @@ int GzipFile(fname, gzipped_fname)
          }
       }
       pclose(pfp);
+      if (!watch_cursor) {
+         SetDefaultCursor(mainWindow);
+         ShowCursor();
+      }
       fclose(out_fp);
       if (writeFileFailed) {
          FailToWriteFileMessage(gzipped_fname);
@@ -567,6 +603,13 @@ char *GunzipFileIntoTemp(gzipped_fname)
    } else {
       sprintf(cmd, uncompress_cmd, gzipped_fname);
    }
+   if (!FindProgramInPath(cmd, NULL, FALSE)) {
+      fclose(out_fp);
+      unlink(tmp_fname);
+      free(cmd);
+      RestoreStatusStrings();
+      return NULL;
+   }
    sprintf(gszMsgBox, TgLoadCachedString(CSTID_EXECUTING_GIVEN_PROGRAM),
          gunzipCmd);
    if (PRTGIF) {
@@ -585,8 +628,12 @@ char *GunzipFileIntoTemp(gzipped_fname)
       unlink(tmp_fname);
       tmp_fname = NULL;
    } else {
-      int bytes_read=0;
+      int bytes_read=0, watch_cursor=watchCursorOnMainWindow;
 
+      if (!watch_cursor) {
+         SetWatchCursor(drawWindow);
+         SetWatchCursor(mainWindow);
+      }
       writeFileFailed = FALSE;
       while ((bytes_read=fread(gszMsgBox, sizeof(char), sizeof(gszMsgBox),
             pfp)) > 0) {
@@ -596,6 +643,10 @@ char *GunzipFileIntoTemp(gzipped_fname)
          }
       }
       pclose(pfp);
+      if (!watch_cursor) {
+         SetDefaultCursor(mainWindow);
+         ShowCursor();
+      }
       fclose(out_fp);
       if (writeFileFailed) {
          FailToWriteFileMessage(tmp_fname);
@@ -640,8 +691,7 @@ int FindXApplResDir(psz_srch_path, psz_target_dir, cb_target_dir)
 static
 void InitLocale()
 {
-   DIR *dirp=NULL;
-   char package[80], *c_ptr=NULL, locale_dir[MAXPATHLENGTH];
+   char *c_ptr=NULL;
 
    c_ptr = MySetLocale(LC_ALL, "");
    if (c_ptr == NULL) {
@@ -661,7 +711,6 @@ void InitLocale()
             }
          }
       } else {
-#ifndef USE_XAPPLRESDIR
 #ifdef USE_XT_INITIALIZE
          if (XtSetLanguageProc(NULL, NULL, NULL) == NULL) {
             fprintf(stderr, "%s\n",
@@ -669,7 +718,6 @@ void InitLocale()
             MySetLocale(LC_ALL, "C");
          }
 #endif /* USE_XT_INITIALIZE */
-#endif /* ~USE_XAPPLRESDIR */
       }
    }
    MySetLocale(LC_ALL, NULL);
@@ -681,6 +729,13 @@ void InitLocale()
     * From this point on, setlocale() should NEVER be called again!
     */
    setlocale(LC_NUMERIC, "C");
+}
+
+static
+void InitLocale2()
+{
+   DIR *dirp=NULL;
+   char package[80], *c_ptr=NULL, locale_dir[MAXPATHLENGTH];
 
    *locale_dir = '\0';
    if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"LocaleDir")) != NULL) {
@@ -841,14 +896,15 @@ int JustLoadAFile(pszFile, pnNeedToCheckAutoExec, nAttemptToMerge, pnAbortMerge)
       } else {
          sprintf(full_name, "%s%c%s", curDir, DIR_SEP, file_name);
       }
-      if (obj_file) {
-         SetCurDir(full_name);
-         *curSymDir = '\0';
-      } else {
-         SetCurSymDir(full_name);
+      if (!cmdLineMerge) {
+         if (obj_file) {
+            SetCurDir(full_name);
+            *curSymDir = '\0';
+         } else {
+            SetCurSymDir(full_name);
+         }
+         curFileDefined = TRUE;
       }
-      curFileDefined = TRUE;
-
       SetFileModified(FALSE);
       sprintf(gszMsgBox, TgLoadString(STID_CUR_EMPTY_FILE_IS),
             (short_name ? rest : file_name));
@@ -937,7 +993,9 @@ int JustLoadAFile(pszFile, pnNeedToCheckAutoExec, nAttemptToMerge, pnAbortMerge)
 
          curFileDefined = TRUE;
          if (file_is_remote) {
-            SetCurDir(remote_fname);
+            if (!cmdLineMerge) {
+               SetCurDir(remote_fname);
+            }
             if (!foundGoodStateObject) PasteString(remote_buf, TRUE, TRUE);
          } else {
             if (*gzipped_fname == '\0') {
@@ -954,11 +1012,13 @@ int JustLoadAFile(pszFile, pnNeedToCheckAutoExec, nAttemptToMerge, pnAbortMerge)
                         gzipped_fname);
                }
             }
-            if (obj_file) {
-               SetCurDir(full_name);
-               *curSymDir = '\0';
-            } else {
-               SetCurSymDir(full_name);
+            if (!cmdLineMerge) {
+               if (obj_file) {
+                  SetCurDir(full_name);
+                  *curSymDir = '\0';
+               } else {
+                  SetCurSymDir(full_name);
+               }
             }
          }
          if (file_is_remote) {
@@ -1030,19 +1090,30 @@ int JustLoadAFile(pszFile, pnNeedToCheckAutoExec, nAttemptToMerge, pnAbortMerge)
    return TRUE;
 }
 
+static
+void SigchldHandler(sig)
+   int sig;
+{
+   int status;
+
+   /* wait for all the zombie processes to terminate */
+   while(waitpid(-1, &status, WNOHANG) > 0);
+}
+
 int JustInit(pszFile, pnNeedToCheckAutoExec)
    char *pszFile;
    int *pnNeedToCheckAutoExec;
 {
-   char *c_ptr=NULL;
-   int cannot_opendisplay=FALSE;
+   int cannot_opendisplay=FALSE, allow_open=TRUE;
+
+#ifdef USE_XT_INITIALIZE
+   useXlib = cmdLineUseXlib;
+#else /* ~USE_XT_INITIALIZE */
 #ifdef USE_XAPPLRESDIR
    FILE *xenv_fp=NULL;
-   char tmp_buf[MAXPATHLENGTH+1];
+   char *c_ptr=NULL, *xEnvStr=NULL, tmp_buf[MAXPATHLENGTH+1];
    int len=0;
-#endif /* USE_XAPPLRESDIR */
 
-#ifdef USE_XAPPLRESDIR
 #ifdef XAPPLOADDIR_DEF
    if (getenv("XENVIRONMENT") == NULL) {
       if (((c_ptr=getenv("XAPPLRESDIR")) == NULL)) {
@@ -1079,14 +1150,17 @@ int JustInit(pszFile, pnNeedToCheckAutoExec)
          }
       }
    }
-   if ((mainDisplay=XOpenDisplay(displayName)) == 0)
-#else /* ~USE_XAPPLRESDIR */
-   c_ptr = NULL;
-#ifdef USE_XT_INITIALIZE
-   useXlib = cmdLineUseXlib;
-#else /* ~USE_XT_INITIALIZE */
+#endif /* USE_XAPPLRESDIR */
    useXlib = TRUE;
 #endif /* USE_XT_INITIALIZE */
+
+#ifndef _NO_LOCALE_SUPPORT
+   MySetLocale(LC_ALL, "C");
+#ifdef ENABLE_NLS
+   InitLocale();
+#endif /* ENABLE_NLS */
+#endif /* ~_NO_LOCALE_SUPPORT */
+
    if (useXlib) {
       cannot_opendisplay = ((mainDisplay=XOpenDisplay(displayName)) == 0);
    } else {
@@ -1097,7 +1171,6 @@ int JustInit(pszFile, pnNeedToCheckAutoExec)
             (mainDisplay=XtDisplay(toplevel)) == 0);
 #endif /* USE_XT_INITIALIZE */
    }
-#endif /* USE_XAPPLRESDIR */
    if (cannot_opendisplay) {
       if (displayName == NULL) {
          fprintf(stderr, "%s\n", TgLoadString(STID_CANT_OPEN_DISPLAY_ABORT));
@@ -1133,20 +1206,25 @@ int JustInit(pszFile, pnNeedToCheckAutoExec)
    signal(SIGBUS, EmergencySave);
 #endif /* ~linux */
    signal(SIGSEGV, EmergencySave);
+   signal(SIGCHLD, SigchldHandler);
 
 #ifndef _NO_LOCALE_SUPPORT
-   MySetLocale(LC_ALL, "C");
 #ifdef ENABLE_NLS
-   InitLocale();
+   InitLocale2();
 #endif /* ENABLE_NLS */
 #endif /* ~_NO_LOCALE_SUPPORT */
 
+   InitWhiteBoard();
+
    Setup();
    InitExtraWinInfo();
+   InitLastKeyOrBtnEvInfo();
    quitDraw = FALSE;
    prTgifFoundColorInfo = FALSE;
 
+   InitWhiteBoard2();
    if (pszFile == NULL) {
+      /* Only gets here if PRTGIF of UrlDump() */
       DelAllPages();
       lastPageNum = 1;
       InitPage();
@@ -1154,10 +1232,51 @@ int JustInit(pszFile, pnNeedToCheckAutoExec)
    }
    initializingMain = TRUE;
 
+   if (gstWBInfo.do_whiteboard) {
+      static int initialized=FALSE;
+
+#ifdef _TGIF_DBG /* debug, do not translate */
+      TgAssert(!initialized,
+            "Whiteboard initialized too many times in JustInit()", NULL);
+#endif /* _TGIF_DBG */
+      if (gstWBInfo.join_session_in_progress) {
+         allow_open = FALSE;
+      }
+      initialized = TRUE;
+   }
+   if (!allow_open) {
+      char *fname=NULL;
+
+      *gszMsgBox = '\0';
+      if (cmdLineMerge && cmdLineMergeArgc > 0 && cmdLineMergeArgv != NULL) {
+         fname = cmdLineMergeArgv[0];
+         sprintf(gszMsgBox, TgLoadString(STID_JOIN_WB_IN_PROGRESS_IGNORED),
+               fname);
+         cmdLineMerge = FALSE;
+      } else if (*pszFile != '\0') {
+         fname = pszFile;
+         sprintf(gszMsgBox, TgLoadString(STID_JOIN_WB_IN_PROGRESS_IGNORED),
+               fname);
+         *pszFile = '\0';
+      }
+      if (fname != NULL) {
+         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+         UtilFree(fname);
+      }
+   }
    if (cmdLineMerge && cmdLineMergeArgc > 0 && cmdLineMergeArgv != NULL) {
-      int arg_index=0, abort_merge=FALSE;
+      int arg_index=0, abort_merge=FALSE, num_argc=cmdLineMergeArgc;
+      int identical=FALSE;
+      char **ppsz_html_export_template=(char**)malloc(cmdLineMergeArgc*
+            sizeof(char*));
+
+      if (ppsz_html_export_template == NULL) FailAllocMessage();
+      memset(ppsz_html_export_template, 0, cmdLineMergeArgc*sizeof(char*));
 
       if (JustLoadAFile(cmdLineMergeArgv[0], NULL, TRUE, &abort_merge)) {
+         ppsz_html_export_template[0] = UtilStrDup(gszHhtmlExportTemplate);
+         if (ppsz_html_export_template[0] == NULL) FailAllocMessage();
+
          if (!abort_merge) {
             PageInfo pi;
 
@@ -1178,6 +1297,11 @@ int JustInit(pszFile, pnNeedToCheckAutoExec)
                      NULL)) {
                   break;
                }
+               ppsz_html_export_template[arg_index] =
+                     UtilStrDup(gszHhtmlExportTemplate);
+               if (ppsz_html_export_template[arg_index] == NULL) {
+                  FailAllocMessage();
+               }
                CleanUpComments();
 
                firstPage->prev = pi.last_page;
@@ -1211,12 +1335,40 @@ int JustInit(pszFile, pnNeedToCheckAutoExec)
       CleanUpMergeArgs();
       ClearFileInfo(FALSE);
       SetFileModified(TRUE);
+
+      identical = TRUE;
+      for (arg_index=1; arg_index < num_argc; arg_index++) {
+         if (strcmp(ppsz_html_export_template[0],
+               ppsz_html_export_template[arg_index]) != 0) {
+            identical = FALSE;
+            break;
+         }
+      }
+      if (!identical) {
+         UtilStrCpyN(gszMsgBox, sizeof(gszMsgBox),
+               TgLoadString(STID_NO_HTML_TEMPLATE_FILE));
+         Msg(gszMsgBox);
+      } else if (*ppsz_html_export_template[0] != '\0') {
+         UtilStrCpyN(gszHhtmlExportTemplate, sizeof(gszHhtmlExportTemplate),
+            ppsz_html_export_template[0]);
+         sprintf(gszMsgBox, TgLoadString(STID_HTML_TEMPLATE_SET_TO_GIVEN),
+               gszHhtmlExportTemplate);
+         Msg(gszMsgBox);
+      }
+      for (arg_index=0; arg_index < num_argc; arg_index++) {
+         UtilFree(ppsz_html_export_template[arg_index]);
+      }
+      free(ppsz_html_export_template);
    } else if (*pszFile != '\0') {
       JustLoadAFile(pszFile, pnNeedToCheckAutoExec, FALSE, NULL);
    } else {
-      DelAllPages();
-      lastPageNum = 1;
-      InitPage();
+      if (allow_open) {
+         DelAllPages();
+         lastPageNum = 1;
+         InitPage();
+      } else {
+         SetFileModified(TRUE);
+      }
    }
    SetDefaultDrawWinClipRecs();
    UpdateDirInfo();
@@ -1228,32 +1380,31 @@ int JustInit(pszFile, pnNeedToCheckAutoExec)
 
 void HandleDataInMBuff()
 {
-#ifdef _TGIF_WB2
    if (gstWBInfo.do_whiteboard) {
       TryHandleWBInputData();
    }
-#endif /* _TGIF_WB2 */
 }
 
 static
 int GetAnXEvent(pXEvent)
    XEvent *pXEvent;
 {
-#ifdef _TGIF_WB2
-   int rc=TRUE;
+   if (gstWBInfo.do_whiteboard) {
+      int rc=TRUE;
 
-   gstWBInfo.BlockRemoteCmdDepth--;
+      gstWBInfo.BlockRemoteCmdDepth--;
 #ifdef DEBUG0
-   fprintf(stderr,"BlockRemoteCmdDepth (GetAnXEvent): %d\n",
-         gstWBInfo.BlockRemoteCmdDepth);
+      fprintf(stderr,"BlockRemoteCmdDepth (GetAnXEvent): %d\n",
+            gstWBInfo.BlockRemoteCmdDepth);
 #endif /* DEBUG0 */
-   if (gnInputMethod != TGIM_NONE && tgIMExpectNextEvent(mainDisplay,
-         drawWindow)) {
-      if (!tgIMHandleNextEvent(mainDisplay, drawWindow, pXEvent)) {
-      }
-   } else {
       if (XPending(mainDisplay)) {
-         XNextEvent(mainDisplay, pXEvent);
+         if (gnInputMethod != TGIM_NONE && tgIMExpectNextEvent(mainDisplay,
+               drawWindow)) {
+            if (!tgIMHandleNextEvent(mainDisplay, drawWindow, pXEvent)) {
+            }
+         } else {
+            XNextEvent(mainDisplay, pXEvent);
+         }
       } else {
          struct timeval timeout;
          fd_set fdset;
@@ -1273,9 +1424,14 @@ int GetAnXEvent(pXEvent)
          status = select(nfds, &fdset, NULL, NULL, &timeout);
 #endif /* __hpux */
          if (status < 0) {
-            sprintf(gszMsgBox, TgLoadString(STID_FUNC_SELECT_SYS_CALL_FAILED),
-                  "WaitForEvent()"),
-            fprintf(stderr, "%s\n", gszMsgBox);
+            if (errno == EINTR) {
+               /* interrupted by a system call, not a problem */
+            } else {
+               sprintf(gszMsgBox,
+                     TgLoadString(STID_FUNC_SELECT_SYS_CALL_FAILED),
+                     "GetAnXEvent()"),
+               fprintf(stderr, "%s\n", gszMsgBox);
+            }
             rc = FALSE;
          } else if (status == 0) {
             /* timeout */
@@ -1300,14 +1456,20 @@ int GetAnXEvent(pXEvent)
                      HandleNewUserRequest();
                   } else {
                      sprintf(gszMsgBox,
-                           TgLoadString(STID_READ_FROM_SELF_PIPE_FAIL), 1,
+                           TgLoadString(STID_UNRECOG_BYTE_FROM_SELF_PIPE), *buf,
                            TOOL_NAME),
                      fprintf(stderr, "%s\n", gszMsgBox);
                   }
                }
                rc = FALSE;
             } else if (FD_ISSET(select_width-1, &fdset)) {
-               XNextEvent(mainDisplay, pXEvent);
+               if (gnInputMethod != TGIM_NONE && tgIMExpectNextEvent(mainDisplay,
+                     drawWindow)) {
+                  if (!tgIMHandleNextEvent(mainDisplay, drawWindow, pXEvent)) {
+                  }
+               } else {
+                  XNextEvent(mainDisplay, pXEvent);
+               }
             } else {
                rc = FALSE;
             }
@@ -1318,25 +1480,21 @@ int GetAnXEvent(pXEvent)
             rc = FALSE;
          }
       }
-   }
-   gstWBInfo.BlockRemoteCmdDepth++;
+      gstWBInfo.BlockRemoteCmdDepth++;
 #ifdef DEBUG0
-   fprintf(stderr,"BlockRemoteCmdDepth (GetAnXEvent): %d\n",
-         gstWBInfo.BlockRemoteCmdDepth);
+      fprintf(stderr,"BlockRemoteCmdDepth (GetAnXEvent): %d\n",
+            gstWBInfo.BlockRemoteCmdDepth);
 #endif /* DEBUG0 */
-   if (!rc) return FALSE;
-#endif /* _TGIF_WB2 */
-
-#if !defined(_TGIF_WB) && !defined(_TGIF_WB2)
-   if (gnInputMethod != TGIM_NONE && tgIMExpectNextEvent(mainDisplay,
-         drawWindow)) {
-      if (!tgIMHandleNextEvent(mainDisplay, drawWindow, pXEvent)) {
-      }
+      if (!rc) return FALSE;
    } else {
-      XNextEvent(mainDisplay, pXEvent);
+      if (gnInputMethod != TGIM_NONE && tgIMExpectNextEvent(mainDisplay,
+            drawWindow)) {
+         if (!tgIMHandleNextEvent(mainDisplay, drawWindow, pXEvent)) {
+         }
+      } else {
+         XNextEvent(mainDisplay, pXEvent);
+      }
    }
-#endif /* !defined(_TGIF_WB) && !defined(_TGIF_WB2) */
-
    return TRUE;
 }
 
@@ -1344,7 +1502,8 @@ int TryProcessAnXEvent(pnNeedToCheckAutoExec)
    int *pnNeedToCheckAutoExec;
 {
    XEvent input;
-   int rc=0;
+   int rc=0, drawtext_cursorshown_keypress_processed=FALSE;
+   TidgetInfo *handling_pti=NULL;
 
    exitNormally = FALSE;
    if (pnNeedToCheckAutoExec != NULL && (*pnNeedToCheckAutoExec) &&
@@ -1360,17 +1519,28 @@ int TryProcessAnXEvent(pnNeedToCheckAutoExec)
       return 0;
    }
 
+   /* set the lastKeyOrBtnEvInfo */
+   if (input.type == KeyPress) {
+      SetLastKeyEvInfo(&input.xkey);
+   } else if (input.type == ButtonPress) {
+      SetLastBtnEvInfo(&input.xbutton);
+   }
+
    if (input.type == FocusIn) {
-      if (canvasFontDoubleByte && curChoice == DRAWTEXT && textCursorShown) {
-         tgIMFocusIn(mainDisplay, mainWindow);
+      if (curChoice == DRAWTEXT && textCursorShown) {
+         tgIMFocusIn(mainDisplay, input.xany.window);
       }
    } else if (input.type == FocusOut) {
-      if (canvasFontDoubleByte && curChoice == DRAWTEXT && textCursorShown) {
-         tgIMFocusOut(mainDisplay, mainWindow);
+      if (curChoice == DRAWTEXT && textCursorShown) {
+         tgIMFocusOut(mainDisplay, input.xany.window);
       }
    } else if (input.type == KeyRelease) {
       return 0;
    } else if (input.type == KeyPress) {
+      if (TidgetManagerWantAllKeyPressEvents()) {
+         TidgetManagerHandleAllKeyPressEvent(&input);
+         return 0;
+      }
       rc = ShortHand(&input);
       switch (rc) {
       case BAD: /* <CONTROL> or <META> */ return 0;
@@ -1388,22 +1558,21 @@ int TryProcessAnXEvent(pnNeedToCheckAutoExec)
          return rc;
       }
    } else if (input.xany.window == drawWindow) {
-#ifdef _TGIF_WB2
       if (input.type == ButtonPress) {
          if (gstWBInfo.do_whiteboard) {
             gstWBInfo.BlockRemoteCmdDepth++;
          }
       }
-#endif /* _TGIF_WB2 */
+      if (curChoice == DRAWTEXT && textCursorShown && input.type == KeyPress) {
+          drawtext_cursorshown_keypress_processed = TRUE;
+      }
       rc = DrawingEventHandler(&input);
-#ifdef _TGIF_WB2
       if (input.type == ButtonPress) {
          if (gstWBInfo.do_whiteboard) {
             gstWBInfo.BlockRemoteCmdDepth--;
             TryHandleWBInputData();
          }
       }
-#endif /* _TGIF_WB2 */
       if (rc != INVALID) {
          return rc;
       }
@@ -1436,6 +1605,11 @@ int TryProcessAnXEvent(pnNeedToCheckAutoExec)
       UserRedrawEventHandler(&input);
    } else if (statusWindow != None && input.xany.window == statusWindow) {
       StatusEventHandler(&input);
+   } else if (chatWindow != None && IsChatWindowEvent(&input, &handling_pti)) {
+      ChatEventHandler(&input, handling_pti);
+      if (curChoice == DRAWTEXT && textCursorShown && input.type == KeyPress) {
+          drawtext_cursorshown_keypress_processed = TRUE;
+      }
    } else if (modeWindow != None && input.xany.window == modeWindow) {
       if ((rc=ModeEventHandler(&input)) != INVALID) {
          return rc;
@@ -1453,6 +1627,8 @@ int TryProcessAnXEvent(pnNeedToCheckAutoExec)
       ColorEventHandler(&input);
    } else if (input.xany.window == colorDummyWindow) {
       ColorDummyEventHandler(&input);
+   } else if (TidgetManagerHandleEvent(&input)) {
+      /* input has been handled by the TidgetManager */
    } else {
       int i=0;
 
@@ -1467,6 +1643,15 @@ int TryProcessAnXEvent(pnNeedToCheckAutoExec)
          }
       }
    }
+#ifndef _NO_ENTER_TEXT_EVERYWHERE
+   if (curChoice == DRAWTEXT && textCursorShown && input.type == KeyPress &&
+         !drawtext_cursorshown_keypress_processed) {
+      rc = DrawingEventHandler(&input);
+      if (rc != INVALID) {
+         return rc;
+      }
+   }
+#endif /* ~_NO_ENTER_TEXT_EVERYWHERE */
    return 0;
 }
 
@@ -1500,7 +1685,7 @@ int TryProcessAnAbortXEvent(esc_to_abort, pf_abort_callback, p_void)
       } else if (!esc_to_abort && has_ch) {
          unsigned int state=key_ev->state;
 
-         if ((state & ControlMask) && (!(state & Mod1Mask)) && s[0] == '\003') {
+         if ((state & ControlMask) && (!(state & METAMASK)) && s[0] == '\003') {
             if (pf_abort_callback == NULL || (pf_abort_callback)(p_void)) {
                return TRUE;
             }
@@ -1583,12 +1768,10 @@ void MainLoop(Op, FileName, FuncStr, Str1, Menu1, Str2, Menu2, Str3, Menu3)
          XCloseDisplay(mainDisplay);
          mainDisplay = NULL;
       } else {
-#ifndef USE_XAPPLRESDIR
 #ifdef USE_XT_INITIALIZE
          XtDestroyWidget(toplevel);
          toplevel = (Widget)0;
 #endif /* USE_XT_INITIALIZE */
-#endif /* ~USE_XAPPLRESDIR */
       }
       exitNormally = TRUE;
       AllocStrings(FuncStr, Str1, Menu1, Str2, Menu2, Str3, Menu3);
@@ -1915,3 +2098,32 @@ void UpdAttrVal(ObjId, AttrName, AttrColor, AttrVal, ReturnStr)
       }
    }
 }
+
+void SetLastKeyEvInfo(key_ev)
+    XKeyEvent *key_ev;
+{
+   lastKeyOrBtnEvInfo.type = key_ev->type;
+   lastKeyOrBtnEvInfo.send_event = key_ev->send_event;
+   lastKeyOrBtnEvInfo.display = key_ev->display;
+   lastKeyOrBtnEvInfo.window = key_ev->window;
+   memcpy(&lastKeyOrBtnEvInfo.time, &key_ev->time, sizeof(Time));
+   lastKeyOrBtnEvInfo.x = key_ev->x;
+   lastKeyOrBtnEvInfo.y = key_ev->y;
+   lastKeyOrBtnEvInfo.keycode_or_button = key_ev->keycode;
+   lastKeyOrBtnEvInfo.state = key_ev->state;
+}
+
+void SetLastBtnEvInfo(button_ev)
+    XButtonEvent *button_ev;
+{
+   lastKeyOrBtnEvInfo.type = button_ev->type;
+   lastKeyOrBtnEvInfo.send_event = button_ev->send_event;
+   lastKeyOrBtnEvInfo.display = button_ev->display;
+   lastKeyOrBtnEvInfo.window = button_ev->window;
+   memcpy(&lastKeyOrBtnEvInfo.time, &button_ev->time, sizeof(Time));
+   lastKeyOrBtnEvInfo.x = button_ev->x;
+   lastKeyOrBtnEvInfo.y = button_ev->y;
+   lastKeyOrBtnEvInfo.keycode_or_button = button_ev->button;
+   lastKeyOrBtnEvInfo.state = button_ev->state;
+}
+
