@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2006, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/menuinfo.c,v 1.9 2006/06/12 23:59:07 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/menuinfo.c,v 1.45 2009/07/20 16:58:03 william Exp $
  */
 
 #define _INCLUDE_FROM_MENUINFO_C_
@@ -44,6 +44,7 @@
 #include "shape.e"
 #include "special.e"
 #include "stream.e"
+#include "stretch.e"
 #include "tangram2.e"
 #include "text.e"
 #include "xbitmap.e"
@@ -118,6 +119,8 @@ static TgMenuItemInfo lineTypeMenuItemInfo[] = {
      N_("Open/closed splines"), NULL, CMDID_CHANGEALLSELLINETYPE },
    { (char*)(&lineTypePixmap[LT_INTSPLINE]), NULL,
      N_("Interpolated splines"), NULL, CMDID_CHANGEALLSELLINETYPE },
+   { (char*)(&lineTypePixmap[LT_STRUCT_SPLINE]), NULL,
+     N_("Structured splines"), NULL, CMDID_CHANGEALLSELLINETYPE },
    { NULL, NULL, NULL, NULL, INVALID }
 };
 TgMenuInfo lineTypeMenuInfo={ TGMUTYPE_BITMAP, lineTypeMenuItemInfo,
@@ -189,16 +192,17 @@ TgMenuInfo stretchableTextModeMenuInfo={ TGMUTYPE_BITMAP,
                                          CreateStretchableTextModeMenu };
 
 static TgMenuItemInfo objShadowMenuItemInfo[] = {
+   { N_("Add Object Shadow"), NULL,
+     N_("Add shadows to selected objects"), NULL, CMDID_ADDOBJSHADOW },
+   { N_("Remove Object Shadow"), NULL,
+     N_("Remove shadows from selected objects"), NULL, CMDID_REMOVEOBJSHADOW },
+   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Set Object Shadow Color..."), NULL,
      N_("Set the shadow color to be used for creating object shadows"), NULL,
      CMDID_SETOBJSHADOWCOLOR },
    { N_("Set Object Shadow Offsets..."), NULL,
      N_("Set the x and y offset for generic object shadow"), NULL,
      CMDID_SETOBJSHADOWOFFSETS },
-   { N_("Add Object Shadow"), NULL,
-     N_("Add shadows to selected objects"), NULL, CMDID_ADDOBJSHADOW },
-   { N_("Remove Object Shadow"), NULL,
-     N_("Remove shadows from selected objects"), NULL, CMDID_REMOVEOBJSHADOW },
    { NULL, NULL, NULL, NULL, INVALID }
 };
 static TgMenuInfo objShadowMenuInfo={ TGMUTYPE_TEXT,
@@ -348,15 +352,19 @@ static TgMenuItemInfo reduceColorMenuItemInfo[] = {
    { N_("Set Default Color Levels..."), NULL,
      N_("Set the number of R G B color levels for the ``standard colors''"),
      NULL, CMDID_SETDEFAULTCOLORLEVELS },
-   { N_("Reduce To Pixmap Colors..."), NULL,
-     N_("Reduce the colors of a selected image to a the colors in an XPM file"),
-     NULL, CMDID_REDUCETOPIXMAPCOLORS },
    { N_("Reduce To Default Colors"), NULL,
      N_("Reduce the colors of a selected image to standard colors"), NULL,
      CMDID_REDUCETODEFAULTCOLORs },
    { N_("Default Error Diffuse"), NULL,
      N_("Error diffuse to reduce colors of a selected image to standard colors"),
      NULL, CMDID_DEFAULTERRORDIFFUSE },
+   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
+   { N_("Reduce To Pixmap Colors..."), NULL,
+     N_("Reduce the colors of a selected image to a the colors in an XPM file"),
+     NULL, CMDID_REDUCETOPIXMAPCOLORS },
+   { N_("Reduce To Mobileweb Safe Colors"), NULL,
+     N_("Reduce the colors of a selected image to colors that are \"MobileWeb safe\" (6 shades of red/green/blue each)"),
+     NULL, CMDID_REDUCETOMOBILEWEBCOLORS },
    { NULL, NULL, NULL, NULL, INVALID }
 };
 static TgMenuInfo reduceColorMenuInfo={ TGMUTYPE_TEXT, reduceColorMenuItemInfo,
@@ -793,6 +801,8 @@ static TgMenuItemInfo polyMenuItemInfo[] = {
      CMDID_DELETEPOINT },
    { N_("Add Point"), "Alt+Cntrl+a",
      N_("Add vertices to a polygon/polygon/spline"), NULL, CMDID_ADDPOINT },
+   { N_("Extend Segment"), NULL,
+     N_("Extend or shrink a line segment"), NULL, CMDID_EXTENDSEGMENT },
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Convert Interpolated Spline"), NULL,
      N_("Convert selected interpolated splines to regular splines"), NULL,
@@ -800,6 +810,12 @@ static TgMenuItemInfo polyMenuItemInfo[] = {
    { N_("Smooth<->Hinge"), NULL,
      N_("Toggle smooth/hinge vertices for a selected poly/polygon/spline"),
      NULL, CMDID_TOGGLESMOOTHHINGE },
+#ifdef NOT_DEFINED
+   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
+   { N_("Use Tighter Structured Splines"), NULL,
+     N_("Toggle between tighter and looser structured splines"),
+     NULL, CMDID_TOGGLETIGHTERSPLINES },
+#endif /* NOT_DEFINED */
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Convert To Bezier Curve"), NULL,
      N_("Convert selected splines to Bezier curves"), NULL,
@@ -821,8 +837,8 @@ static TgMenuItemInfo polyMenuItemInfo[] = {
      N_("Add Tick Marks submenu >>>"), &addTickMarksMenuInfo, INVALID },
    { NULL, NULL, NULL, NULL, INVALID }
 };
-static TgMenuInfo polyMenuInfo={ TGMUTYPE_TEXT, polyMenuItemInfo,
-                                 TgCreateMenuFromMenuInfo };
+TgMenuInfo polyMenuInfo={ TGMUTYPE_TEXT, polyMenuItemInfo,
+                                 CreatePolySplineMenu };
 
 static TgMenuItemInfo textMenuItemInfo[] = {
    { N_("Break Up Text"), "Alt+#",
@@ -896,6 +912,9 @@ static TgMenuItemInfo createObjectMenuItemInfo[] = {
    { N_("Get Bounding Box"), NULL,
      N_("Get rectangular bounding boxes of selected objects"), NULL,
      CMDID_GETBOUNDINGBOX },
+   { N_("Create Pixmap From Selected"), NULL,
+     N_("Create an X11 pixmap object from selected objects"), NULL,
+     CMDID_CREATEPIXMAPFROMSEL },
    { NULL, NULL, NULL, NULL, INVALID }
 };
 static TgMenuInfo createObjectMenuInfo={ TGMUTYPE_TEXT,
@@ -931,6 +950,13 @@ static TgMenuItemInfo editMenuItemInfo[] = {
      N_("Select a file and paste the content as text"), NULL,
      CMDID_PASTEFROMFILE },
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
+   { N_("Copy Double-byte (UTF-8) String"), NULL,
+     N_("Copy selected double-byte text string into the cut buffer (UTF-8) encoding"), NULL,
+     CMDID_COPYUTF8 },
+   { N_("Paste Double-byte (UTF-8) String"), NULL,
+     N_("Paste from the cut buffer (UTF-8 encoding) into a double-byte text object"), NULL,
+     CMDID_PASTEUTF8 },
+   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Generic Object Shadow"), TGMUITEM_SUBMENU,
      N_("Generic Object Shadow submenu >>>"), &objShadowMenuInfo, INVALID },
    { N_("Shape"), TGMUITEM_PINNABLESUBMENU,
@@ -1037,6 +1063,52 @@ static TgMenuInfo alignToPageDirectMenuInfo={ TGMUTYPE_BITMAP, NULL,
 static TgMenuInfo distrObjsDirectMenuInfo={ TGMUTYPE_BITMAP, NULL,
                                             CreateDistributeDirectMenu };
 
+static TgMenuItemInfo autoRotatePivotMenuItemInfo[] = {
+   { (char*)(&rotateModePixmap[AUTO_ROTATE_PIVOT]), NULL,
+     N_("Set rotation pivot mode to automatic"),
+     NULL, CMDID_CHOOSEROTATEPIVOT },
+   { (char*)(&rotateModePixmap[USER_ROTATE_PIVOT]), NULL,
+     N_("Set rotation pivot mode to user-specified"),
+     NULL, CMDID_CHOOSEROTATEPIVOT },
+   { NULL, NULL, NULL, NULL, INVALID }
+};
+TgMenuInfo autoRotatePivotMenuInfo={ TGMUTYPE_BITMAP,
+                                     autoRotatePivotMenuItemInfo,
+                                     CreateAutoRotatePivotMenu };
+
+static TgMenuItemInfo moveStdRotatePivotMenuItemInfo[] = {
+   { N_("Move Rotation Pivot To Center"), NULL,
+     N_("Move rotation pivot to the center of selected objects"), NULL,
+     CMDID_MOVEROTATEPIVOTCENTER },
+   { N_("Move Rotation Pivot To LeftTop Corner"), NULL,
+     N_("Move rotation pivot to the left-top corner of selected objects"),
+     NULL, CMDID_MOVEROTATEPIVOTLT },
+   { N_("Move Rotation Pivot To RightTop Corner"), NULL,
+     N_("Move rotation pivot to the right-top corner of selected objects"),
+     NULL, CMDID_MOVEROTATEPIVOTRT },
+   { N_("Move Rotation Pivot To LeftBottom Corner"), NULL,
+     N_("Move rotation pivot to the left-bottom corner of selected objects"),
+     NULL, CMDID_MOVEROTATEPIVOTLB },
+   { N_("Move Rotation Pivot To RightBottom Corner"), NULL,
+     N_("Move rotation pivot to the right-bottom corner of selected objects"),
+     NULL, CMDID_MOVEROTATEPIVOTRB },
+   { N_("Move Rotation Pivot To Left Center"), NULL,
+     N_("Move rotation pivot to the left center of selected objects"),
+     NULL, CMDID_MOVEROTATEPIVOTLEFT },
+   { N_("Move Rotation Pivot To Right Center"), NULL,
+     N_("Move rotation pivot to the right center of selected objects"),
+     NULL, CMDID_MOVEROTATEPIVOTRIGHT },
+   { N_("Move Rotation Pivot To Top Center"), NULL,
+     N_("Move rotation pivot to the top center of selected objects"),
+     NULL, CMDID_MOVEROTATEPIVOTTOP },
+   { N_("Move Rotation Pivot To Bottom Center"), NULL,
+     N_("Move rotation pivot to the bottom center of selected objects"),
+     NULL, CMDID_MOVEROTATEPIVOTBOTTOM },
+   { NULL, NULL, NULL, NULL, INVALID }
+};
+TgMenuInfo rotatePivotMenuInfo={ TGMUTYPE_TEXT, moveStdRotatePivotMenuItemInfo,
+                                CreateMoveStdRotatePivotMenu };
+
 static TgMenuItemInfo flipRotateMenuItemInfo[] = {
    { N_("Flip Horizontal"), "Alt+h",
      N_("Flip selected objects horizontally"), NULL, CMDID_FLIPHORIZONTAL },
@@ -1054,10 +1126,29 @@ static TgMenuItemInfo flipRotateMenuItemInfo[] = {
    { N_("Set Rotation Increment..."), NULL,
      N_("Set rotation increment in degrees"), NULL,
      CMDID_SETROTATIONINCREMENT },
+   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
+   { N_("Rotation Pivot"), TGMUITEM_SUBMENU,
+     N_("Rotation pivot submenu >>>"), &autoRotatePivotMenuInfo, INVALID },
+   { N_("Reset Rotation Pivot"), NULL,
+     N_("Reset user-specified rotation pivot to center of selected objects"),
+     NULL, CMDID_RESETROTATEPIVOT },
+   { N_("Speficy Rotation Pivot..."), NULL,
+     N_("Specify the coordinates for the user-specified rotation pivot"), NULL,
+     CMDID_SPECIFYROTATEPIVOT },
+   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
+   { N_("Next Poly/Polygon Rotation Pivot"), NULL,
+     N_("Move the rotation pivot to the next vertex of a poly/polygon object"),
+     NULL, CMDID_NEXTPOLYROTATEPIVOT },
+   { N_("Move Rotation Pivot To Arc Center"), NULL,
+     N_("Move the rotation pivot to the center of the circle which an arc object is part of"),
+     NULL, CMDID_MOVEROTATEPIVOTARCCNTR },
+   { N_("Move Standard Rotation Pivot"), TGMUITEM_SUBMENU,
+     N_("Move standard rotation pivot submenu >>>"), &rotatePivotMenuInfo,
+     INVALID },
    { NULL, NULL, NULL, NULL, INVALID }
 };
 TgMenuInfo flipRotateMenuInfo={ TGMUTYPE_TEXT, flipRotateMenuItemInfo,
-                                TgCreateMenuFromMenuInfo };
+                                CreateFlipRotateMenu };
 
 static TgMenuItemInfo arrangeMenuItemInfo[] = {
    { N_("Front"), "Cntrl+f", N_("Bring selected objects to the front"), NULL,
@@ -1161,10 +1252,8 @@ static TgMenuItemInfo showHideMenuItemInfo[] = {
      CMDID_TOGGLESHOWSTATUS },
    { N_("Show Mode"), NULL, N_("Show mode window"), NULL,
      CMDID_TOGGLESHOWMODE },
-#ifdef _TGIF_WB2
    { N_("Show Chat"), NULL, N_("Show chat window"), NULL,
      CMDID_TOGGLESHOWCHAT },
-#endif /* _TGIF_WB2 */
    { NULL, NULL, NULL, NULL, INVALID }
 };
 static TgMenuInfo showHideMenuInfo={ TGMUTYPE_TEXT, showHideMenuItemInfo,
@@ -1319,6 +1408,9 @@ static TgMenuItemInfo propertiesMenuItemInfo[] = {
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Add Color..."), NULL, N_("Add colors to the current palette"), NULL,
      CMDID_ADDCOLOR },
+   { N_("Choose Color..."), NULL,
+     N_("Choose a color from a color wheel and add it to the current palette"),
+     NULL, CMDID_COLORWHEEL },
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Set Edit Text Size..."), NULL,
      N_("Set the text size to be used in editing existing text objects"), NULL,
@@ -1747,10 +1839,8 @@ static TgMenuItemInfo mainMenuItemInfo[] = {
      N_("Navigate Menu"), &navigateMenuInfo, MENU_NAVIGATE },
    { N_("Special"), TGMUITEM_PINNABLESUBMENU,
      N_("Special Menu"), &specialMenuInfo, MENU_SPECIAL },
-#ifdef _TANGRAM_II
    { N_("Tangram-II"), TGMUITEM_PINNABLESUBMENU,
      N_("Tangram-II Menu"), &tangram2MenuInfo, MENU_TANGRAM2 },
-#endif /* _TANGRAM_II */
    { N_("Help"), TGMUITEM_PINNABLESUBMENU, N_("Help Menu"), &helpMenuInfo,
      MENU_HELP },
    { NULL, NULL, NULL, NULL, INVALID }
@@ -1780,10 +1870,8 @@ static TgMenuItemInfo minimalMenubarMenuItemInfo[] = {
      N_("Navigate Menu"), &navigateMenuInfo, MENU_NAVIGATE },
    { N_("Special"), TGMUITEM_PINNABLESUBMENU,
      N_("Special Menu"), &specialMenuInfo, MENU_SPECIAL },
-#ifdef _TANGRAM_II
    { N_("Tangram-II"), TGMUITEM_PINNABLESUBMENU,
      N_("Tangram-II Menu"), &tangram2MenuInfo, MENU_TANGRAM2 },
-#endif /* _TANGRAM_II */
    { N_("Help"), TGMUITEM_PINNABLESUBMENU, N_("Help Menu"), &helpMenuInfo,
      MENU_HELP },
    { NULL, NULL, NULL, NULL, INVALID }
@@ -1844,10 +1932,8 @@ static TgMenuItemInfo maximalMenubarMenuItemInfo[] = {
      N_("Navigate Menu"), &navigateMenuInfo, MENU_NAVIGATE },
    { N_("Special"), TGMUITEM_PINNABLESUBMENU,
      N_("Special Menu"), &specialMenuInfo, MENU_SPECIAL },
-#ifdef _TANGRAM_II
    { N_("Tangram-II"), TGMUITEM_PINNABLESUBMENU,
      N_("Tangram-II Menu"), &tangram2MenuInfo, MENU_TANGRAM2 },
-#endif /* _TANGRAM_II */
    { N_("Help"), TGMUITEM_PINNABLESUBMENU, N_("Help Menu"), &helpMenuInfo,
      MENU_HELP },
    { NULL, NULL, NULL, NULL, INVALID }
