@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/msg.c,v 4.57 2001/10/15 20:54:10 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/msg.c,v 1.44 2009/10/02 20:14:03 william Exp $
  */
 
 #define _INCLUDE_FROM_MSG_C_
@@ -23,7 +23,9 @@
 #include "tgifdefs.h"
 #include "patchlvl.h"
 
+#include "auxtext.e"
 #include "button.e"
+#include "chat.e"
 #include "cutpaste.e"
 #include "cursor.e"
 #include "dialog.e"
@@ -42,8 +44,11 @@
 #include "raster.e"
 #include "rect.e"
 #include "remote.e"
+#include "rm_intrf.e"
 #include "setup.e"
 #include "strtbl.e"
+#include "tangram2.e"
+#include "text.e"
 #include "util.e"
 #include "version.e"
 #include "wb.e"
@@ -59,6 +64,10 @@ char	progName[MAXPATHLENGTH+1];
 char	fatalMsgBuf1[MAXSTRING<<1];
 char	fatalMsgBuf2[MAXSTRING<<1];
 
+int	cmdLineParentWindowID=0;
+int	cmdLineTgrm2=FALSE;
+int	cmdLineWb2=FALSE;
+
 struct MsgRec {
    char			* s;
    struct MsgRec	* next, * prev;
@@ -199,19 +208,6 @@ int TgAssert(must_be_true, msg1, msg2)
    return FALSE;
 }
 
-int BoldMsgDoubleByte(pszStr)
-   char *pszStr;
-{
-#ifdef ENABLE_NLS
-   if (boldMsgFontDoubleByte) {
-      if ((*pszStr) & 0x80) {
-         return TRUE;
-      }
-   }
-#endif /* ENABLE_NLS */
-   return FALSE;
-}
-
 int BoldMsgCharBytes(pszStr)
    char *pszStr;
 {
@@ -351,6 +347,32 @@ int BoldMsgTextWidth(font_ptr, psz_str, len)
    return XTextWidth(font_ptr, psz_str, len);
 }
 
+int ItalicMsgTextWidth(font_ptr, psz_str, len)
+   XFontStruct *font_ptr;
+   char *psz_str;
+   int len;
+{
+#ifdef ENABLE_NLS
+   if (italicMsgFontSet != NULL) {
+      return XmbTextEscapement(italicMsgFontSet, psz_str, len);
+   }
+#endif /* ENABLE_NLS */
+   return XTextWidth(font_ptr, psz_str, len);
+}
+
+int BoldItalicMsgTextWidth(font_ptr, psz_str, len)
+   XFontStruct *font_ptr;
+   char *psz_str;
+   int len;
+{
+#ifdef ENABLE_NLS
+   if (boldItalicMsgFontSet != NULL) {
+      return XmbTextEscapement(boldItalicMsgFontSet, psz_str, len);
+   }
+#endif /* ENABLE_NLS */
+   return XTextWidth(font_ptr, psz_str, len);
+}
+
 void DrawMenuString(dpy, win, gc, x, baseline_y, menu_str, len)
    Display *dpy;
    Window win;
@@ -399,6 +421,40 @@ void DrawBoldMsgString(dpy, win, gc, x, baseline_y, msg_str, len)
    XDrawString(dpy, win, gc, x, baseline_y, msg_str, len);
 }
 
+void DrawItalicMsgString(dpy, win, gc, x, baseline_y, msg_str, len)
+   Display *dpy;
+   Window win;
+   GC gc;
+   int x, baseline_y, len;
+   char *msg_str;
+{
+#ifdef ENABLE_NLS
+   if (italicMsgFontSet != NULL) {
+      XmbDrawString(dpy, win, italicMsgFontSet, gc, x, baseline_y, msg_str,
+            len);
+      return;
+   }
+#endif /* ENABLE_NLS */
+   XDrawString(dpy, win, gc, x, baseline_y, msg_str, len);
+}
+
+void DrawBoldItalicMsgString(dpy, win, gc, x, baseline_y, msg_str, len)
+   Display *dpy;
+   Window win;
+   GC gc;
+   int x, baseline_y, len;
+   char *msg_str;
+{
+#ifdef ENABLE_NLS
+   if (boldItalicMsgFontSet != NULL) {
+      XmbDrawString(dpy, win, boldItalicMsgFontSet, gc, x, baseline_y, msg_str,
+            len);
+      return;
+   }
+#endif /* ENABLE_NLS */
+   XDrawString(dpy, win, gc, x, baseline_y, msg_str, len);
+}
+
 static
 void AddMsg(Msg)
    char *Msg;
@@ -826,37 +882,30 @@ void SetStringStatus(StatusStr)
    }
 }
 
-typedef struct StatusInfoRec {
-   char btn_str[MAX_STATUS_BTNS][MAXSTRING+1];
-   char one_line_str[MAXSTRING+1];
-   int one_line_status;
-   struct StatusInfoRec *next;
-} *StatusInfoPtr;
-
-static struct StatusInfoRec *topStatusInfo=NULL;
+static MouseOverStatusInfo *topStatusInfo=NULL;
 
 void SaveStatusStrings()
 {
-   struct StatusInfoRec *psi;
-   int i;
+   MouseOverStatusInfo *pmosi=NULL;
+   int i=0;
 
    if (PRTGIF || noStatusWindow) return;
 
-   psi = (struct StatusInfoRec *)malloc(sizeof(struct StatusInfoRec));
-   if (psi == NULL) FailAllocMessage();
-   psi->next = topStatusInfo;
+   pmosi = (MouseOverStatusInfo *)malloc(sizeof(MouseOverStatusInfo));
+   if (pmosi == NULL) FailAllocMessage();
+   pmosi->next = topStatusInfo;
    for (i=0; i < MAX_STATUS_BTNS; i++) {
-      UtilStrCpyN(psi->btn_str[i], MAXSTRING+1, btnStatusStr[i]);
+      UtilStrCpyN(pmosi->btn_str[i], MAXSTRING+1, btnStatusStr[i]);
    }
-   UtilStrCpyN(psi->one_line_str, MAXSTRING+1, oneLineStatusStr);
-   psi->one_line_status = oneLineStatus;
-   topStatusInfo = psi;
+   UtilStrCpyN(pmosi->one_line_str, MAXSTRING+1, oneLineStatusStr);
+   pmosi->one_line_status = oneLineStatus;
+   topStatusInfo = pmosi;
 }
 
 void RestoreStatusStrings()
 {
-   struct StatusInfoRec *psi;
-   int i;
+   MouseOverStatusInfo *pmosi=NULL;
+   int i=0;
 
    if (PRTGIF || noStatusWindow || topStatusInfo == NULL) return;
 
@@ -873,9 +922,9 @@ void RestoreStatusStrings()
    } else {
       RedrawStatusWindow();
    }
-   psi = topStatusInfo->next;
+   pmosi = topStatusInfo->next;
    free(topStatusInfo);
-   topStatusInfo = psi;
+   topStatusInfo = pmosi;
 }
 
 void SaveStatusStringsIntoBuf(ppsz_buf, pn_one_line)
@@ -916,6 +965,37 @@ void RestoreStatusStringsFromBuf(ppsz_buf, one_line)
    }
 }
 
+void BeginProgress(ppi, total_size)
+   ProgressInfo *ppi;
+   int total_size;
+{
+   memset(ppi, 0, sizeof(ProgressInfo));
+   ppi->total_size = total_size;
+   SaveStatusStringsIntoBuf(ppi->status_buf, &ppi->one_line_status);
+}
+
+void UpdateProgress(ppi, cur_size)
+   ProgressInfo *ppi;
+   int cur_size;
+{
+   int percent=(int)(((double)cur_size) / ((double)ppi->total_size) *
+            ((double)100));
+
+   if (percent >= ppi->target_percent) {
+      sprintf(gszMsgBox, TgLoadCachedString(CSTID_PROGRESS_PERCENT),
+            percent);
+      SetStringStatus(gszMsgBox);
+      XSync(mainDisplay, False);
+      while (ppi->target_percent <= percent) ppi->target_percent += 5;
+   }
+}
+
+void EndProgress(ppi)
+   ProgressInfo *ppi;
+{
+   RestoreStatusStringsFromBuf(ppi->status_buf, ppi->one_line_status);
+}
+
 void MsgEventHandler(input)
    XEvent *input;
 {
@@ -1108,13 +1188,13 @@ void InitStatus()
 void CleanUpStatus()
 {
    while (topStatusInfo != NULL) {
-      struct StatusInfoRec *next_si=topStatusInfo->next;
+      MouseOverStatusInfo *next_mosi=topStatusInfo->next;
 
       free(topStatusInfo);
-      topStatusInfo = next_si;
+      topStatusInfo = next_mosi;
    }
-   if (redraw_bits) { }
-   if (btn1_bits) { }
+   if (redraw_bits == NULL) { }
+   if (btn1_bits == NULL) { }
 }
 
 static char	*scanVal=NULL;
@@ -1188,6 +1268,60 @@ int ScanValue(fmt, v, item, stype)
    return 0;
 }
 
+int ScanDynStrValue(v, item, stype)
+   struct DynStrRec *v;
+   char *item, *stype;
+   /*
+    * 
+    */
+{
+   char *c_ptr=NULL, *psz=NULL, msg[MAXPATHLENGTH];
+
+   if (scanVal == NULL) return INVALID;
+#ifdef DEBUG /* debug, do not translate */
+   printf("get %s for %s from %s:  ", item, stype, scanVal);
+#endif
+
+   if ((c_ptr=GetString()) == NULL) {
+#ifdef DEBUG /* debug, do not translate */
+      printf("nothing\n");
+#endif
+      (void)sprintf(msg, TgLoadString(STID_FILE_LINE_MISS_FIELD_IN_OBJ),
+            scanFileName, scanLineNum, item, stype);
+      if (PRTGIF) {
+         fprintf(stderr, "%s\n", msg);
+      } else {
+         Msg(msg);
+      }
+      return INVALID;
+   }
+   while (*c_ptr != '"' && *c_ptr != '\0') c_ptr++;
+   if (c_ptr != NULL) {
+      psz = ReadString(++c_ptr);
+   }
+   if (psz == NULL) {
+#ifdef DEBUG /* debug, do not translate */
+      printf("error in %s\n", c_ptr);
+#endif
+      (void)sprintf(msg, TgLoadString(STID_FILE_LINE_BAD_FIELD_IN_OBJ),
+            scanFileName, scanLineNum, item, stype, c_ptr);
+      if (PRTGIF) {
+         fprintf(stderr, "%s\n", msg);
+      } else {
+         Msg(msg);
+      }
+      return INVALID;
+   }
+   *(--psz) = '\0';
+   DynStrSet(v, c_ptr);
+
+#ifdef DEBUG /* debug, do not translate */
+   printf("got \"%s\"\n", ((struct DynStrRec *)v)->s);
+#endif
+
+   return 0;
+}
+
 #define MAXEMERGENCYCOUNT 5
 
 static int	emergencyCount = 0;
@@ -1276,6 +1410,7 @@ int EmergencySaveForXCont(s)
    if (exitNormally) return 0;
 
    if (copyingToCutBuffer) {
+      emergencyCount--;
       copyingToCutBuffer = INVALID;
       return 0;
    }
@@ -1356,6 +1491,9 @@ void GetTgifVersionAndPatchLevel(buf, buf_sz)
             TGIF_DISTR_VER);
    }
 #endif /* TGIF_DISTR_VER */
+#ifdef NOT_DEFINED
+   sprintf(&buf1[strlen(buf1)], TgLoadString(STID_TGIF_FLAGS), "_TGIF_WB2");
+#endif /* NOT_DEFINED */
    UtilStrCpyN(buf, buf_sz, buf1);
 }
 
@@ -1365,59 +1503,72 @@ void Usage(tool_name)
    char buf[MAXSTRING];
 
    GetTgifVersionAndPatchLevel(buf, sizeof(buf));
-   fprintf(stderr, "%s\n", buf);
+   fprintf(stdout, "%s\n", buf);
 
-   fprintf(stderr, "%s\n\n", copyrightString);
-   fprintf(stderr, "Usage:\t%s \\\n", tool_name);
-   fprintf(stderr, "\t[-display displayname] \\\n");
-   fprintf(stderr, "\t[-fg <color>] \\\n");
-   fprintf(stderr, "\t[-bg <color>] \\\n");
-   fprintf(stderr, "\t[-bd <color>] \\\n");
-   fprintf(stderr, "\t[-rv] [-nv] \\\n");
-   fprintf(stderr, "\t[-bw] \\\n");
-   fprintf(stderr, "\t[-reqcolor] \\\n");
-   fprintf(stderr, "\t[-cwo[+sbwarp]] \\\n");
-   fprintf(stderr, "\t[-hyper] \\\n");
-   fprintf(stderr, "\t[-a4] \\\n");
-   fprintf(stderr, "\t[-geometry <geom>] [=<geom>] \\\n");
-   fprintf(stderr, "\t[-exec <file>] \\\n");
-   fprintf(stderr, "\t[-dbim {xcin|chinput|xim|kinput2}] \\\n");
-   fprintf(stderr, "\t[-usexlib] \\\n");
-   fprintf(stderr, "\t[-listdontreencode] \\\n");
-   fprintf(stderr, "\t[{<file>[.%s] | -merge <file1>[.%s] <file2>[.%s] ...}]\n",
+   fprintf(stdout, "%s\n\n", copyrightString);
+   fprintf(stdout, "Usage:\t%s \\\n", tool_name);
+   fprintf(stdout, "\t[-display displayname] \\\n");
+   fprintf(stdout, "\t[-fg <color>] \\\n");
+   fprintf(stdout, "\t[-bg <color>] \\\n");
+   fprintf(stdout, "\t[-bd <color>] \\\n");
+   fprintf(stdout, "\t[-rv] [-nv] \\\n");
+   fprintf(stdout, "\t[-bw] \\\n");
+   fprintf(stdout, "\t[-tgwb2 [-rmcastlibdir <directory> | -rmcastlibpath <path>] [-parent <parent_window_id>]] \\\n");
+   fprintf(stdout, "\t[-reqcolor] \\\n");
+   fprintf(stdout, "\t[-cwo[+sbwarp]] \\\n");
+   fprintf(stdout, "\t[-hyper] \\\n");
+   fprintf(stdout, "\t[-a4] \\\n");
+   fprintf(stdout, "\t[-geometry <geom>] [=<geom>] \\\n");
+   fprintf(stdout, "\t[-exec <file>] \\\n");
+   fprintf(stdout, "\t[-dbim {xcin|chinput|xim|kinput2}] \\\n");
+   fprintf(stdout, "\t[-sbim xim] \\\n");
+   fprintf(stdout, "\t[-usexlib] \\\n");
+   fprintf(stdout, "\t[-listdontreencode] \\\n");
+   fprintf(stdout, "\t[-version | --version] \\\n");
+   fprintf(stdout, "\t[-pdfspd | -pdfspd=true | -pdfspd=false] \\\n");
+   fprintf(stdout, "\t[-pssetup \"<string>\"] \\\n");
+   fprintf(stdout, "\t[-nomode] \\\n");
+   fprintf(stdout, "\t[-tg2genparse=\"<string>\"] \\\n");
+   fprintf(stdout, "\t[{<file>[.%s] | -merge <file1>[.%s] <file2>[.%s] ...}]\n",
          OBJ_FILE_EXT, OBJ_FILE_EXT, OBJ_FILE_EXT);
-   fprintf(stderr, "or:\n");
-   fprintf(stderr, "\t%s -print \\\n", tool_name);
-   fprintf(stderr, "\t[-display displayname] \\\n");
-   fprintf(stderr, "\t[-stdout] \\\n");
-   fprintf(stderr, "\t[-eps | -p | -epsi | -tiffepsi] \\\n");
-   fprintf(stderr, "\t[-ps | -f] \\\n");
-   fprintf(stderr, "\t[-text] \\\n");
-   fprintf(stderr, "\t[-gif | -xpm | -xbm | -html] \\\n");
-   fprintf(stderr, "\t[-raw[+h[eaderonly]]] \\\n");
-   fprintf(stderr, "\t[-dosepsfilter [-previewonly]] \\\n");
-   fprintf(stderr, "\t[-status] \\\n");
-   fprintf(stderr, "\t[-gray] \\\n");
-   fprintf(stderr, "\t[-color | -reqcolor] \\\n");
-   fprintf(stderr, "\t[-adobe | -adobe=<number>/<number>] \\\n");
-   fprintf(stderr,
+   fprintf(stdout, "or:\n");
+   fprintf(stdout, "\t%s -print \\\n", tool_name);
+   fprintf(stdout, "\t[-display displayname] \\\n");
+   fprintf(stdout, "\t[-eps | -p | -epsi | -tiffepsi] \\\n");
+   fprintf(stdout, "\t[-ps | -f] \\\n");
+   fprintf(stdout, "\t[-text] \\\n");
+   fprintf(stdout, "\t[-gif | -png | -jpeg | -ppm | -pbm | -xpm | -xbm | -html] \\\n");
+   fprintf(stdout, "\t[-pdf] \\\n");
+   fprintf(stdout, "\t[-netlist] \\\n");
+   fprintf(stdout, "\t[-stdout] \\\n");
+   fprintf(stdout, "\t[-raw[+h[eaderonly]]] \\\n");
+   fprintf(stdout, "\t[-dosepsfilter [-previewonly]] \\\n");
+   fprintf(stdout, "\t[-status] \\\n");
+   fprintf(stdout, "\t[-gray] \\\n");
+   fprintf(stdout, "\t[-color | -reqcolor] \\\n");
+   fprintf(stdout, "\t[-adobe | -adobe=<number>/<number>] \\\n");
+   fprintf(stdout,
          "\t[-dontreencode=\"<%s>:<%s>:...\" | -listdontreencode] \\\n",
          "fontfamily", "fontfamily");
-   fprintf(stderr, "\t[-producedby=\"<string>\"] \\\n");
-   fprintf(stderr, "\t[-page <number>] \\\n");
-   fprintf(stderr, "\t[-print_cmd \"<command>\"] \\\n");
-   fprintf(stderr, "\t[-one_file_per_page] \\\n");
-   fprintf(stderr, "\t[-pepsc] \\\n");
-   fprintf(stderr, "\t[-nolandpdfspd] \\\n");
-   fprintf(stderr, "\t[-dontcondense | -condensed] \\\n");
-   fprintf(stderr, "\t[-a4] \\\n");
-   fprintf(stderr, "\t[-quiet] \\\n");
-   fprintf(stderr, "\t[-bop_hook \"<string>\"] \\\n");
-   fprintf(stderr, "\t[-eop_hook \"<string>\"] \\\n");
-   fprintf(stderr, "\t[-tmp_file_mode \"<octal number>\"] \\\n");
-   fprintf(stderr, "\t[-odir] \\\n");
-   fprintf(stderr, "\t[-exec <file>] \\\n");
-   fprintf(stderr, "\t[<file1>[.%s] <file2>[.%s] ...]\n",
+   fprintf(stdout, "\t[-version | --version | -justversion] \\\n");
+   fprintf(stdout, "\t[-producedby=\"<string>\"] \\\n");
+   fprintf(stdout, "\t[-page <number>] \\\n");
+   fprintf(stdout, "\t[-print_cmd \"<command>\"] \\\n");
+   fprintf(stdout, "\t[-one_file_per_page] \\\n");
+   fprintf(stdout, "\t[-pepsc] \\\n");
+   fprintf(stdout, "\t[-pdfspd | -pdfspd=true | -pdfspd=false] \\\n");
+   fprintf(stdout, "\t[-pssetup \"<string>\"] \\\n");
+   fprintf(stdout, "\t[-j2p6_cmd \"<command>\"] \\\n");
+   fprintf(stdout, "\t[-dontcondense | -condensed] \\\n");
+   fprintf(stdout, "\t[-a4] \\\n");
+   fprintf(stdout, "\t[-noshowpageineps] \\\n");
+   fprintf(stdout, "\t[-quiet] \\\n");
+   fprintf(stdout, "\t[-bop_hook \"<string>\"] \\\n");
+   fprintf(stdout, "\t[-eop_hook \"<string>\"] \\\n");
+   fprintf(stdout, "\t[-tmp_file_mode \"<octal number>\"] \\\n");
+   fprintf(stdout, "\t[-odir] \\\n");
+   fprintf(stdout, "\t[-exec <file>] \\\n");
+   fprintf(stdout, "\t[<file1>[.%s] <file2>[.%s] ...]\n",
          OBJ_FILE_EXT, OBJ_FILE_EXT);
    exit(-1);
 }
@@ -1427,26 +1578,32 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
    char *argv[];
 {
    char *c_ptr=NULL;
-   int saved_argc=argc, show_copyright=TRUE;
+   int saved_argc=argc, show_copyright=FALSE;
    char **saved_argv=argv;
 
    *cmdLineTmpFileModeStr = '\0';
+   *cmdLineGenParser = '\0';
+   *cmdLineFileToExec = '\0';
+   *cmdLineJpegToPpm6Cmd = '\0';
+   *cmdLineCustomPatternDir = '\0';
    cmdLineOneFilePerPage = FALSE;
    *outputDir = '\0';
    for (argc--, argv++; argc > 0; argc--, argv++) {
       if (**argv == '-') {
-         if (strcmp(*argv, "-raw") == 0) {
-            show_copyright = FALSE;
-         } else if (strcmp(*argv, "-raw+h") == 0) {
-            show_copyright = FALSE;
-         } else if (strcmp(*argv, "-raw+headeronly") == 0) {
-            show_copyright = FALSE;
-         } else if (strcmp(*argv, "-stdout") == 0) {
-            show_copyright = FALSE;
-         } else if (strcmp(*argv, "-parse_html") == 0) {
-            show_copyright = FALSE;
-         } else if (strcmp(*argv, "-quiet") == 0) {
-            show_copyright = FALSE;
+         if (strcmp(*argv, "-version") == 0 ||
+               strcmp(*argv, "--version") == 0) {
+            show_copyright = TRUE;
+         } else if (strcmp(*argv, "-tgrm2") == 0) {
+            cmdLineTgrm2 = TRUE;
+         } else if (strcmp(*argv, "-justversion") == 0) {
+            char buf[MAXSTRING];
+
+            GetTgifVersionAndPatchLevel(buf, sizeof(buf));
+            fprintf(stdout, "%s\n", buf);
+            fprintf(stdout, "%s (%s)\n\n", copyrightString, authorEmailString);
+            fflush(stdout);
+
+            return INVALID;
          }
       }
    }
@@ -1456,7 +1613,7 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
       char buf[MAXSTRING];
 
       GetTgifVersionAndPatchLevel(buf, sizeof(buf));
-      fprintf(stderr, "%s", buf);
+      fprintf(stderr, "%s\n", buf);
       fprintf(stderr, "%s (%s)\n\n", copyrightString, authorEmailString);
       fflush(stderr);
    }
@@ -1510,12 +1667,13 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
    c_ptr += strlen(printCommand);
    for (argc--, argv++; argc > 0; argc--, argv++) {
       if (**argv == '-') {
-         int len;
+         int len=0;
 
          if (strcmp(*argv, "-display") == 0) {
             argc--; argv++;
             if (argc <= 0) {
-               fprintf(stderr, "%s\n", TgLoadString(STID_MISSING_DISPLAY_NAME));
+               fprintf(stderr, "%s\n\n",
+                     TgLoadString(STID_MISSING_DISPLAY_NAME));
                Usage(progName);
             }
             if ((displayName=UtilStrDup(*argv)) == NULL) FailAllocMessage();
@@ -1546,6 +1704,14 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             cmdLineWhereToPrint = JPEG_FILE;
             cmdLineColor = TRUE;
             cmdLineOpenDisplay = TRUE;
+         } else if (strcmp(*argv,"-ppm") == 0) {
+            cmdLineWhereToPrint = PPM_FILE;
+            cmdLineColor = TRUE;
+            cmdLineOpenDisplay = TRUE;
+         } else if (strcmp(*argv,"-pbm") == 0) {
+            cmdLineWhereToPrint = PPM_FILE;
+            cmdLineColor = FALSE;
+            cmdLineOpenDisplay = TRUE;
          } else if (strcmp(*argv,"-xpm") == 0) {
             cmdLineWhereToPrint = XBM_FILE;
             cmdLineColor = TRUE;
@@ -1564,6 +1730,8 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             whereToPrint = cmdLineWhereToPrint = TEXT_FILE;
          } else if (strcmp(*argv, "-netlist") == 0) {
             whereToPrint = cmdLineWhereToPrint = NETLIST_FILE;
+         } else if (strcmp(*argv,"-svg") == 0) {
+            whereToPrint = cmdLineWhereToPrint = SVG_FILE;
          } else if (strcmp(*argv, "-raw") == 0) {
             cmdLineDumpURL = TRUE;
             cmdLineDumpURLWithHeader = FALSE;
@@ -1585,7 +1753,8 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
          } else if (strcmp(*argv, "-page") == 0) {
             argc--; argv++;
             if (argc <= 0) {
-               fprintf(stderr, "%s\n", TgLoadString(STID_MISSING_PAGE_NUMBER));
+               fprintf(stderr, "%s\n\n",
+                     TgLoadString(STID_MISSING_PAGE_NUMBER));
                Usage(progName);
             }
             cmdLineHasPageNum = TRUE;
@@ -1595,7 +1764,7 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
                cmdLineHasPageNum = FALSE;
                fprintf(stderr, TgLoadString(STID_INVALID_GIVEN_PAGE_NUMBER_STR),
                      cmdLinePageNumStr);
-               fprintf(stderr, "\n");
+               fprintf(stderr, "\n\n");
                Usage(progName);
             }
          } else if (strcmp(*argv, "-one_file_per_page") == 0) {
@@ -1603,13 +1772,44 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
          } else if (strcmp(*argv, "-pepsc") == 0) {
             stripEPSComments = FALSE;
          } else if (strcmp(*argv, "-nolandpdfspd") == 0) {
-            cmdLineNoLandscapePdfSetPageDevice = TRUE;
+            cmdLinePdfSetPageDevice = FALSE;
+            fprintf(stderr, TgLoadString(STID_OBSOLETE_CMDLINE_ASSUME),
+                  "-nolandpdfspd", "-pdfspd=false");
+            fprintf(stderr, "\n");
+         } else if (strcmp(*argv, "-pdfspd") == 0) {
+            cmdLinePdfSetPageDevice = TRUE;
+         } else if (strncmp(*argv, "-pdfspd=", 8) == 0) {
+            char tmp_str[80];
+
+            UtilStrCpyN(tmp_str, sizeof(tmp_str), &(*argv)[8]);
+            UtilTrimBlanks(tmp_str);
+            if (UtilStrICmp(tmp_str, "true") == 0) {
+               cmdLinePdfSetPageDevice = TRUE;
+            } else if (UtilStrICmp(tmp_str, "false") == 0) {
+               cmdLinePdfSetPageDevice = FALSE;
+            } else {
+               fprintf(stderr, TgLoadString(STID_BAD_CMDLINE_OPT_IGNORED),
+                     "-pdfspd", tmp_str);
+               fprintf(stderr, "\n");
+            }
+         } else if (strcmp(*argv, "-pssetup") == 0) {
+            argc--; argv++;
+            if (argc <= 0) {
+               fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
+                     "-pssetup");
+               fprintf(stderr, "\n\n");
+               Usage(progName);
+            }
+            cmdLinePsSetup = TRUE;
+            SetPsSetup(*argv);
          } else if (strcmp(*argv, "-dontcondense") == 0) {
             SetCmdLineDontCondense();
          } else if (strcmp(*argv, "-condensed") == 0) {
             SetCmdLineCondensedPS();
          } else if (strcmp(*argv, "-a4") == 0) {
             cmdLineA4 = TRUE;
+         } else if (strcmp(*argv, "-noshowpageineps") == 0) {
+            cmdLineNoShowPageInEPS = TRUE;
          } else if (strcmp(*argv, "-quiet") == 0) {
             cmdLineQuiet = TRUE;
          } else if (strcmp(*argv, "-bop_hook") == 0) {
@@ -1617,7 +1817,7 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             if (argc <= 0) {
                fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                      "-bop_hook");
-               fprintf(stderr, "\n");
+               fprintf(stderr, "\n\n");
                Usage(progName);
             }
             SetBopHook(*argv);
@@ -1626,7 +1826,7 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             if (argc <= 0) {
                fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                      "-eop_hook");
-               fprintf(stderr, "\n");
+               fprintf(stderr, "\n\n");
                Usage(progName);
             }
             SetEopHook(*argv);
@@ -1637,7 +1837,7 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             if (argc <= 0) {
                fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                      "-tmp_file_mode");
-               fprintf(stderr, "\n");
+               fprintf(stderr, "\n\n");
                Usage(progName);
             }
             UtilStrCpyN(cmdLineTmpFileModeStr, sizeof(cmdLineTmpFileModeStr),
@@ -1651,7 +1851,7 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
                   fprintf(stderr,
                         TgLoadString(STID_INVALID_ARG_FOR_CMDLINE_OPT),
                         "-tmp_file_mode", cmdLineTmpFileModeStr);
-                  fprintf(stderr, "\n");
+                  fprintf(stderr, "\n\n");
                   Usage(progName);
                }
             }
@@ -1673,8 +1873,8 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             UtilTrimBlanks(tmp_str);
             if (!ParsePsAdobeString(tmp_str, &usePsAdobeString, adobeString,
                   epsfString)) {
-               fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR),
-                     TOOL_NAME, "UsePsAdobeString", tmp_str, "false");
+               fprintf(stderr, TgLoadString(STID_BAD_CMDLINE_OPT_USE_ALT_STR),
+                     "-adobe", tmp_str, "false");
                fprintf(stderr, "\n");
                usePsAdobeString = FALSE;
                *adobeString = *epsfString = '\0';
@@ -1683,6 +1883,9 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
 #ifdef _DONT_REENCODE
             fprintf(stderr, "-D_DONT_REENCODE=\"%s\"\n", _DONT_REENCODE);
 #endif /* _DONT_REENCODE */
+         } else if (strcmp(*argv, "-version") == 0 ||
+               strcmp(*argv, "--version") == 0) {
+            cmdLineVersion = TRUE;
          } else if (strncmp(*argv, "-dontreencode=", 14) == 0) {
             strcpy(cmdLineDontReencode, &(*argv)[14]);
             UtilTrimBlanks(cmdLineDontReencode);
@@ -1697,6 +1900,13 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             if (*cmdLineProducedBy != '\0') {
                UtilRemoveQuotes(cmdLineProducedBy);
             }
+         } else if (strncmp(*argv, "-tg2genparser=", 14) == 0) {
+            UtilStrCpyN(cmdLineGenParser, sizeof(cmdLineGenParser),
+                  &(*argv)[14]);
+            UtilTrimBlanks(cmdLineGenParser);
+            if (*cmdLineGenParser != '\0') {
+               UtilRemoveQuotes(cmdLineGenParser);
+            }
          } else if (strcmp(*argv, "-parse_html") == 0) {
             cmdLineParseHtml = TRUE;
             /* cmdLineDumpURL will be reset in UrlDump() */
@@ -1708,13 +1918,14 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             if (argc <= 0) {
                fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                      "-exec");
-               fprintf(stderr, "\n");
+               fprintf(stderr, "\n\n");
                Usage(progName);
             }
             UtilStrCpyN(cmdLineFileToExec, sizeof(cmdLineFileToExec), *argv);
             UtilTrimBlanks(cmdLineFileToExec);
             if (*cmdLineFileToExec == '-' && cmdLineFileToExec[1] != '\0') {
-               fprintf(stderr, "%s\n", TgLoadString(STID_BAD_EXEC_FNAME_DASH));
+               fprintf(stderr, "%s\n\n",
+                     TgLoadString(STID_BAD_EXEC_FNAME_DASH));
                Usage(progName);
             }
             cmdLineOpenDisplay = TRUE;
@@ -1723,7 +1934,7 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             if (argc <= 0) {
                fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                      "-tgif_path");
-               fprintf(stderr, "\n");
+               fprintf(stderr, "\n\n");
                Usage(progName);
             }
             len = strlen(*argv);
@@ -1742,7 +1953,7 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             if (argc <= 0) {
                fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                      "-print_cmd");
-               fprintf(stderr, "\n");
+               fprintf(stderr, "\n\n");
                Usage(progName);
             }
             len = strlen(*argv);
@@ -1752,6 +1963,56 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             } else {
                strcpy(printCommand, *argv);
             }
+         } else if (strcmp(*argv, "-j2p6_cmd") == 0) {
+            argc--; argv++;
+            if (argc <= 0) {
+               fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
+                     "-j2p6_cmd");
+               fprintf(stderr, "\n\n");
+               Usage(progName);
+            }
+            len = strlen(*argv);
+            if (**argv == '"' && (*argv)[len-1] == '"') {
+               if (len-3 >= MAXSTRING) {
+                  fprintf(stderr, TgLoadString(STID_CMDLINE_ARG_GIVEN_TOO_LONG),
+                        "-j2p6_cmd");
+                  fprintf(stderr, "\n");
+               } else {
+                  strcpy(cmdLineJpegToPpm6Cmd, &(*argv)[1]);
+                  cmdLineJpegToPpm6Cmd[strlen(cmdLineJpegToPpm6Cmd)-1] = '\0';
+               }
+            } else {
+               if (len-1 >= MAXSTRING) {
+                  fprintf(stderr, TgLoadString(STID_CMDLINE_ARG_GIVEN_TOO_LONG),
+                        "-j2p6_cmd");
+                  fprintf(stderr, "\n");
+               } else {
+                  strcpy(cmdLineJpegToPpm6Cmd, *argv);
+                  cmdLineJpegToPpm6Cmd[strlen(cmdLineJpegToPpm6Cmd)-1] = '\0';
+               }
+            }
+         } else if (strcmp(*argv, "-patterndir") == 0) {
+            argc--; argv++;
+            if (argc <= 0) {
+               fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
+                     "-patterndir");
+               fprintf(stderr, "\n\n");
+               Usage(progName);
+            }
+            UtilStrCpyN(cmdLineCustomPatternDir,
+                  sizeof(cmdLineCustomPatternDir), *argv);
+            UtilTrimBlanks(cmdLineCustomPatternDir);
+            len = strlen(cmdLineCustomPatternDir);
+            if (len > 0 && cmdLineCustomPatternDir[len-1] == DIR_SEP) {
+               cmdLineCustomPatternDir[len-1] = '\0';
+            }
+            if (*cmdLineCustomPatternDir == '\0' ||
+                  !UtilIsDirectory(cmdLineCustomPatternDir)) {
+               fprintf(stderr, TgLoadString(STID_PATTERNDIR_NOT_EXIST),
+                     cmdLineCustomPatternDir);
+               fprintf(stderr, "\n");
+               *cmdLineCustomPatternDir = '\0';
+            }
          } else if (strncmp(*argv, "-o", 2) == 0) {
             strcpy(outputDir, &(*argv)[2]);
          } else if (strcmp(*argv, "-prtgif") != 0 &&
@@ -1770,7 +2031,7 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             (cmdLineOpenDisplay && cmdLineWhereToPrint == INVALID)) {
          fprintf(stderr, TgLoadString(STID_PRINT_TO_PRINTER_INCOMPAT),
                "-one_file_page_page");
-         fprintf(stderr, "\n");
+         fprintf(stderr, "\n\n");
          Usage(progName);
       }
    }
@@ -1783,21 +2044,63 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
    /* All these strangeness with strings are related to */
    /*	Prolog's foreign function interface. */
 {
-   int i=0;
+   int i=0, len=0;
 
    *file_name = '\0';
    *cmdLineTmpFileModeStr = '\0';
+   *cmdLineGenParser = '\0';
    *cmdLineFileToExec = '\0';
+   *cmdLineRMCastLibDir = *cmdLineRMCastLibPath = '\0';
+   *cmdLineCustomPatternDir = '\0';
    cmdLineMergeArgv = NULL;
    for (i=1; i < argc; i++) {
       if (strcmp(argv[i], "-display") == 0) {
          if (++i == argc) {
-            fprintf(stderr, "%s\n", TgLoadString(STID_MISSING_DISPLAY_NAME));
+            fprintf(stderr, "%s\n\n", TgLoadString(STID_MISSING_DISPLAY_NAME));
             Usage(progName);
          }
          if ((displayName=UtilStrDup(argv[i])) == NULL) FailAllocMessage();
+      } else if (strcmp(argv[i], "-tgrm2") == 0) {
+         cmdLineTgrm2 = TRUE;
+      } else if (strcmp(argv[i], "-rmcastlibdir") == 0) {
+         if (++i == argc) {
+            fprintf(stderr, "%s\n\n", TgLoadString(STID_MISSING_RMCASTDIR));
+            Usage(progName);
+         }
+         UtilStrCpyN(cmdLineRMCastLibDir, sizeof(cmdLineRMCastLibDir), argv[i]);
+      } else if (strcmp(argv[i], "-rmcastlibpath") == 0) {
+         if (++i == argc) {
+            fprintf(stderr, "%s\n\n", TgLoadString(STID_MISSING_RMCASTLIB));
+            Usage(progName);
+         }
+         UtilStrCpyN(cmdLineRMCastLibPath, sizeof(cmdLineRMCastLibPath),
+               argv[i]);
+      } else if (strcmp(argv[i], "-tgwb2") == 0) {
+         cmdLineWb2 = TRUE;
       } else if (strcmp(argv[i], "-bw") == 0) {
          cmdLineBW = TRUE;
+      } else if (strcmp(argv[i], "-parent") == 0) {
+         if (++i == argc) {
+            fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
+                  "-parent");
+            fprintf(stderr, "\n\n");
+            Usage(progName);
+         }
+         if (strncmp(argv[i], "0x", 2) == 0) {
+            if (sscanf(&(argv[i])[2], "%x", &cmdLineParentWindowID) != 1) {
+               fprintf(stderr, TgLoadString(STID_FAIL_TO_PARSE_FOR_A_VAL),
+                     argv[i]);
+               fprintf(stderr, "\n\n");
+               Usage(progName);
+            }
+         } else {
+            if (sscanf(argv[i], "%d", &cmdLineParentWindowID) != 1) {
+               fprintf(stderr, TgLoadString(STID_FAIL_TO_PARSE_FOR_A_VAL),
+                     argv[i]);
+               fprintf(stderr, "\n\n");
+               Usage(progName);
+            }
+         }
       } else if (strcmp(argv[i], "-reqcolor") == 0) {
          cmdLineRequestedColor = TRUE;
       } else if (strcmp(argv[i], "-rv") == 0) {
@@ -1816,43 +2119,71 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
          if (++i == argc) {
             fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                   "-exec");
-            fprintf(stderr, "\n");
+            fprintf(stderr, "\n\n");
             Usage(progName);
          }
          UtilStrCpyN(cmdLineFileToExec, sizeof(cmdLineFileToExec), argv[i]);
          UtilTrimBlanks(cmdLineFileToExec);
          if (*cmdLineFileToExec == '-' && cmdLineFileToExec[1] != '\0') {
-            fprintf(stderr, "%s\n", TgLoadString(STID_BAD_EXEC_FNAME_DASH));
+            fprintf(stderr, "%s\n\n", TgLoadString(STID_BAD_EXEC_FNAME_DASH));
             Usage(progName);
          }
       } else if (strcmp(argv[i], "-dbim") == 0) {
          if (++i == argc) {
             fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                   "-dbim");
-            fprintf(stderr, "\n");
+            fprintf(stderr, "\n\n");
+            Usage(progName);
+         }
+         if (cmdLineHasInputMethod) {
+            fprintf(stderr, "%s\n\n", TgLoadString(STID_ONLYONE_DBIM_OR_SBIM));
             Usage(progName);
          }
          strcpy(cmdLineInputMethod, argv[i]);
          cmdLineHasInputMethod = TRUE;
+         cmdLineDoubleByteInputMethod = TRUE;
+      } else if (strcmp(argv[i], "-sbim") == 0) {
+         if (++i == argc) {
+            fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
+                  "-sbim");
+            fprintf(stderr, "\n\n");
+            Usage(progName);
+         }
+         if (cmdLineHasInputMethod) {
+            fprintf(stderr, "%s\n\n", TgLoadString(STID_ONLYONE_DBIM_OR_SBIM));
+            Usage(progName);
+         }
+         strcpy(cmdLineInputMethod, argv[i]);
+         cmdLineHasInputMethod = TRUE;
+         cmdLineDoubleByteInputMethod = FALSE;
       } else if (strcmp(argv[i], "-usexlib") == 0) {
          cmdLineUseXlib = TRUE;
       } else if (strcmp(argv[i], "-merge") == 0) {
          cmdLineMerge = TRUE;
       } else if (strcmp(argv[i], "-a4") == 0) {
          cmdLineA4 = TRUE;
-#ifdef _TGIF_WB2
 #ifdef NOT_DEFINED
       } else if (strcmp(argv[i], "-listen") == 0) {
          cmdLineWhiteBoardListenOnly = TRUE;
 #endif /* NOT_DEFINED */
       } else if (strcmp(argv[i], "-no-wb") == 0) {
          cmdLineNoWhiteBoard = TRUE;
-#endif /* _TGIF_WB2 */
+      } else if (strcmp(argv[i], "-no-chat") == 0) {
+         cmdLineNoChatWindow = TRUE;
+      } else if (strcmp(argv[i], "-chatnick") == 0) {
+         if (++i == argc) {
+            fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
+                  "-chatnick");
+            fprintf(stderr, "\n\n");
+            Usage(progName);
+         }
+         cmdLineChatNickName = UtilStrDup(argv[i]);
+         if (cmdLineChatNickName == NULL) FailAllocMessage();
       } else if (strcmp(argv[i], "-fg") == 0) {
          if (++i == argc) {
             fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                   "-fg");
-            fprintf(stderr, "\n");
+            fprintf(stderr, "\n\n");
             Usage(progName);
          }
          cmdLineForeground = (char*)malloc(80*sizeof(char));
@@ -1862,7 +2193,7 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
          if (++i == argc) {
             fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                   "-bg");
-            fprintf(stderr, "\n");
+            fprintf(stderr, "\n\n");
             Usage(progName);
          }
          cmdLineBackground = (char*)malloc(80*sizeof(char));
@@ -1872,7 +2203,7 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
          if (++i == argc) {
             fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                   "-bd");
-            fprintf(stderr, "\n");
+            fprintf(stderr, "\n\n");
             Usage(progName);
          }
          cmdLineBorder = (char*)malloc(80*sizeof(char));
@@ -1882,7 +2213,7 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
          if (++i == argc) {
             fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
                   "-geometry");
-            fprintf(stderr, "\n");
+            fprintf(stderr, "\n\n");
             Usage(progName);
          }
          strcat(geometrySpec, argv[i]);
@@ -1891,10 +2222,63 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
 #ifdef _DONT_REENCODE
          fprintf(stderr, "-D_DONT_REENCODE=\"%s\"\n", _DONT_REENCODE);
 #endif /* _DONT_REENCODE */
-      } else if (*argv[i] == '=') {
+      } else if (strcmp(argv[i], "-version") == 0 ||
+            strcmp(argv[i], "--version") == 0) {
+         cmdLineVersion = TRUE;
+      } else if (strcmp(argv[i], "-nomode") == 0) {
+         cmdLineNoModeWindow = TRUE;
+      } else if (strcmp(argv[i], "-pdfspd") == 0) {
+         cmdLinePdfSetPageDevice = TRUE;
+      } else if (strncmp(argv[i], "-pdfspd=", 8) == 0) {
+         char tmp_str[80];
+
+         UtilStrCpyN(tmp_str, sizeof(tmp_str), &(argv[i])[8]);
+         UtilTrimBlanks(tmp_str);
+         if (UtilStrICmp(tmp_str, "true") == 0) {
+            cmdLinePdfSetPageDevice = TRUE;
+         } else if (UtilStrICmp(tmp_str, "false") == 0) {
+            cmdLinePdfSetPageDevice = FALSE;
+         } else {
+            fprintf(stderr, TgLoadString(STID_BAD_CMDLINE_OPT_IGNORED),
+                  "-pdfspd", tmp_str);
+            fprintf(stderr, "\n");
+         }
+      } else if (strcmp(argv[i], "-pssetup") == 0) {
+         i++;
+         if (i >= argc) {
+            fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
+                  "-pssetup");
+            fprintf(stderr, "\n\n");
+            Usage(progName);
+         }
+         cmdLinePsSetup = TRUE;
+         SetPsSetup(argv[i]);
+      } else if (strcmp(*argv, "-patterndir") == 0) {
+         argc--; argv++;
+         if (argc <= 0) {
+            fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
+                  "-patterndir");
+            fprintf(stderr, "\n\n");
+            Usage(progName);
+         }
+         UtilStrCpyN(cmdLineCustomPatternDir,
+               sizeof(cmdLineCustomPatternDir), *argv);
+         UtilTrimBlanks(cmdLineCustomPatternDir);
+         len = strlen(cmdLineCustomPatternDir);
+         if (len > 0 && cmdLineCustomPatternDir[len-1] == DIR_SEP) {
+            cmdLineCustomPatternDir[len-1] = '\0';
+         }
+         if (*cmdLineCustomPatternDir == '\0' ||
+               !UtilIsDirectory(cmdLineCustomPatternDir)) {
+            fprintf(stderr, TgLoadString(STID_PATTERNDIR_NOT_EXIST),
+                  cmdLineCustomPatternDir);
+            fprintf(stderr, "\n");
+            *cmdLineCustomPatternDir = '\0';
+         }
+      } else if (argv[i][0] == '=') {
          strcpy(geometrySpec, argv[i]);
          geometrySpecified = TRUE;
-      } else if (*argv[i] != '-') {
+      } else if (argv[i][0] != '-') {
          if (cmdLineMerge) {
             int j=0;
 
@@ -1913,7 +2297,17 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
             UtilStrCpyN(file_name, name_sz, argv[i]);
          }
       } else {
+#ifndef USE_XT_INITIALIZE
          Usage(progName);
+#else /* USE_XT_INITIALIZE */
+         if (strcmp(argv[i], "-xrm") == 0) {
+            if (++i == argc) {
+               fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
+                     "-xrm");
+               fprintf(stderr, "\n");
+            }
+         }
+#endif /* ~USE_XT_INITIALIZE */
       }
    }
    return TRUE;
